unit Unit2;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  CommPort, VML_Modem;

const

  { Notify message }

  WM_MODEMNOTIFY   = WM_USER + 1;

  { Modem operation state }

  MS_IDLE      = 0; { idle state }
  MS_STARTING  = 1; { initialize modem }
  MS_PLAYING   = 2; { playing voice }
  MS_RECORDING = 3; { recording voice }
  MS_STOPPING  = 4; { stopping current action }
  MS_ERROR     = 5; { error }

type

{ TModemThread }

  TModemOperation = (opPlay, opRecord);

  TModemThread = class(TThread)
  private
    FOwner: TObject;
    FPlayFileName: string;
    FRecordFileName: string;
    FOperation: TModemOperation;
    FErrorMsg: string;
    procedure DoErrorMsg;
    procedure DoNotify(ModemState: Integer);
    procedure DoPlay;
    procedure DoRecord;
  protected
    procedure Call(Method: TThreadMethod);
    function Check(ModemResult: Integer; const ErrorMsg: string): Integer;
    procedure DoTerminate; override;
    procedure Execute; override;
  public
    constructor Create(AOwner: TObject);
    procedure Stop;
  end;

{ TModemModule }

  TModemModule = class(TDataModule)
    VoiceModem: TVoiceModem;
    ModemProfile: TModemProfile;
    ModemLogFile: TModemLogFile;
    ModemTraceFile: TModemLogFile;
    procedure ModemModuleCreate(Sender: TObject);
  private
    FThread: TModemThread;
    FNewParams: Boolean;
    procedure CreateThread(Owner: TObject);
    procedure ThreadDone(Sender: TObject);
  public
    procedure PlayFile(Owner: TObject; const PlayFileName: string);
    procedure RecordFile(Owner: TObject; const RecordFileName: string);
    procedure Stop(Wait: Boolean);
    procedure UpdateParams(IgnoreLocking: Boolean);
  end;

var
  ModemModule: TModemModule;

implementation

uses
  Unit1, VML_IniFile, VML_Consts;

{$R *.DFM}

{ TModemThread }

constructor TModemThread.Create(AOwner: TObject);
begin
  inherited Create(True);
  FOwner := AOwner;
end;

procedure TModemThread.DoErrorMsg;
const
  Flags = MB_OK or MB_ICONERROR;
begin
  with Application do MessageBox(PChar(FErrorMsg), PChar(Title), Flags);
end;

procedure TModemThread.DoNotify(ModemState: Integer);
begin
  PostMessage(TestForm.Handle, WM_MODEMNOTIFY, ModemState, Longint(FOwner));
end;

procedure TModemThread.DoPlay;
const
  Error1 = 'An error occured while setting play device. %s.';
  Error2 = 'An error occured while playing voice file. %s.';
  Error3 = 'An error occured while trying to reset play device. %s.';
  Error4 = 'An error occured while restoring voice mode. %s.';
begin
  with ModemModule.VoiceModem do
  begin
    { Set play device }
    Check(SetDevice(Profile.PlayDevice), Error1);
    { Play file }
    DoNotify(MS_PLAYING);
    Check(PlayFile(FPlayFileName), Error2);
    { Reset device }
    Check(SetDevice(NO_DEVICE), Error3);
    Check(RestoreVoiceMode, Error4);
  end;
end;

procedure TModemThread.DoRecord;
const
  Error1 = 'An error occured while setting record device. %s.';
  Error2 = 'An error occured while recording voice file. %s.';
  Error3 = 'An error occured while trying to reset record device. %s.';
  Error4 = 'An error occured while restoring voice mode. %s.';
begin
  with ModemModule.VoiceModem do
  begin
    { Set record device }
    Check(SetDevice(Profile.RecDevice), Error1);
    { Record voice }
    DoNotify(MS_RECORDING);
    Check(RecordFile(FRecordFileName), Error2);
    { Reset device }
    Check(SetDevice(NO_DEVICE), Error3);
    Check(RestoreVoiceMode, Error4);
  end;
end;

procedure TModemThread.Call(Method: TThreadMethod);
begin
  { In main thread call method without synchronization }
  if GetCurrentThreadID = MainThreadID then
  begin
    Method;
    Exit;
  end;
  { Check thread not terminated and synchronize method }
  if not Terminated then Synchronize(Method);
end;

function TModemThread.Check(ModemResult: Integer; const ErrorMsg: string): Integer;
begin
  Result := ModemResult;
  { Check for modem errors }
  if (ModemResult = S_FAIL) or (ModemResult = VMA_FAIL) then
  begin
    with ModemModule.VoiceModem do
    begin
      { Save modem last error, close modem }
      ReturnValue := LastError;
      Close;
    end;
    { Raise exception }
    raise Exception.CreateFmt(ErrorMsg, [GetVMLErrorStr(ReturnValue)]);
  end;
end;

procedure TModemThread.DoTerminate;
begin
  { Idle state }
  DoNotify(MS_IDLE);
  { Check thread not terminated and call inherited method }
  if not Terminated then inherited DoTerminate;
end;

procedure TModemThread.Execute;
const
  Error1 = 'Could not access modem. Device is busy.';
  Error2 = 'An error occured while initialize modem. %s. Check modem settings and try again.';
begin
  with ModemModule.VoiceModem do
  try
    { Lock modem }
    if not Lock(Self) then
    begin
      ReturnValue := VME_MODEM_LOCKED;
      raise Exception.Create(Error1);
    end;
    try
      { Starting in progress }
      DoNotify(MS_STARTING);
      { Check new settings }
      with ModemModule do
        if FNewParams then UpdateParams(True);
      { Open device }
      Check(Open, Error2);
      { Check modem }
      Check(Command('AT', 'OK'), Error2);
      { Flush modem event queue }
      FlushEvents;
      { Check tread terminated }
      if not Terminated then
      begin
        { Process operation }
        case FOperation of
          opPlay: DoPlay;
          opRecord: DoRecord;
        end;
      end;
    finally
      Unlock;
    end;
  except
    on E: Exception do
    begin
      { Error state }
      DoNotify(MS_ERROR);
      { Show error message }
      FErrorMsg := E.Message;
      Call(DoErrorMsg);
    end;
  end;
end;

procedure TModemThread.Stop;
begin
  { Queue stop event }
  ModemModule.VoiceModem.Stop;
  { Stopping in progress }
  DoNotify(MS_STOPPING);
end;

{ TModemModule }

procedure TModemModule.ThreadDone(Sender: TObject);
begin
  FThread := nil;
end;

procedure TModemModule.CreateThread(Owner: TObject);
const
  Error1 = 'Could not access modem. Device is busy.';
  Flags = MB_OK or MB_ICONERROR;
begin
  { Lock modem }
  if (FThread <> nil) or VoiceModem.Locked then
  begin
    with Application do MessageBox(PChar(Error1), PChar(Title), Flags);
    Abort;
  end;
  { Create operation thread }
  FThread := TModemThread.Create(Owner);
  FThread.FreeOnTerminate := True;
  FThread.OnTerminate := ThreadDone;
end;

procedure TModemModule.PlayFile(Owner: TObject; const PlayFileName: string);
begin
  { Create operation thread }
  CreateThread(Owner);
  { Fill thread params }
  FThread.FPlayFileName := PlayFileName;
  FThread.FOperation := opPlay;
  { Start operation }
  FThread.Resume;
end;

procedure TModemModule.RecordFile(Owner: TObject; const RecordFileName: string);
begin
  { Create operation thread }
  CreateThread(Owner);
  { Fill thread params }
  FThread.FRecordFileName := RecordFileName;
  FThread.FOperation := opRecord;
  { Start operation }
  FThread.Resume;
end;

procedure TModemModule.Stop(Wait: Boolean);
begin
  { Check operation in progress }
  if FThread <> nil then
  begin
    Screen.Cursor := crHourGlass;
    try
      { Stop operation }
      if Wait then
      begin
        VoiceModem.Stop;
        { Wait thread }
        FThread.Terminate;
        FThread.WaitFor;
        { Delete thread link }
        FThread := nil;
      end
      else
        FThread.Stop;
    finally
      Screen.Cursor := crDefault;
    end;
  end;
end;

procedure TModemModule.UpdateParams(IgnoreLocking: Boolean);
begin
  FNewParams := True;
  { Check modem locking }
  if (not VoiceModem.Locked) or IgnoreLocking then
  begin
    { Close device }
    if VoiceModem.Opened then VoiceModem.Close;
    { Update profile settings }
    ReadProfileIniFile('DrvTest.ini', ModemProfile);
    { Chande modem driver }
    VoiceModem.Driver := ReadDriverIniFile(ModemProfile.ModemInifile, nil);
    { Update complete }
    FNewParams := False;
  end;
end;

procedure TModemModule.ModemModuleCreate(Sender: TObject);
begin
  with ModemLogFile do FileName := ExtractFilePath(Application.ExeName) + FileName;
  with ModemTraceFile do FileName := ExtractFilePath(Application.ExeName) + FileName;
  { Update settings }
  UpdateParams(True);
  { Update driveer field }
  Testform.DriverEdit.Text := ModemProfile.ModemIniFile;
end;

end.
