{
  Voice Modem Library v1.0

  This file contains expanded driver for Rockwell modems with
  software answer recognition feature.

  Note. This driver automatically replaces standard Rockwell driver.

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit AR_Rockwell;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, Graphics, VML_Modem, VML_Consts,
  VML_Rockwell, FFTu;

{$I *.INC}

type

{ TARRockwell }

  TARRockwell = class(TRockwell)
  private
    FAnswerStream: TMemoryStream;
    FAnswerPos: Longint;
    FLinBuff: prmas;
    FLincount: Integer;
    FAnalyzing: Boolean;
    FConvertError: Boolean;
    FToneTime: Longint;
    FVoiceTime: Longint;
    FWasVoice: Boolean;
    FAnswer: Integer;
    FTimeOut: Boolean;
    FInterrupted: Boolean;
  protected
    Fa, Fmf, Fte, Fc: Integer;
    procedure DrawSpectrum(Rect: TRect; Canvas: TCanvas);
    procedure Progress(Done: Longint); override;
  public
    constructor Create(AOwner: TComponent); override;
    function Dial(const Number: string): Integer; override;
    function HandleEvent(Event: Integer; Data: TEventData): Integer; override;
  end;

implementation

uses
  MMSys, Unit3;

{ TARRockwell }

constructor TARRockwell.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { Answer recognition method currently work only with PCM 8 bit
    compression becous this is linead data and not required any
    additional conversion. }
  Compressions.Clear;
  Compressions.Add(ROCKWELL_PCM_8, SRockwellPCM8, 'AT#VSR=%d||AT#VBS=8', 'OK||OK', 7200, 8, CF_7200 or CF_11025, WAVE_FORMAT_PCM, 11025, CF_11025 or CF_DEFAULT);
end;

procedure TARRockwell.DrawSpectrum(Rect: TRect; Canvas: TCanvas);
const
  N = 1024;
var
  Width, Height, i: Integer;
  max: Double;
  m: array[1..N] of Word; 

  { Get X coordinate for given value index }

  function xx(x: Integer): Integer;
  begin
    xx := 2 * Round(Width / N * x);
  end;

begin
  max := Emax;
  if max = 0 then max := 1;
  for i := 1 to N div 2 do m[i] := Round(ms[i] / max * 255);

  Width := Rect.Right - Rect.Left;
  Height := Rect.Bottom - Rect.Top;

  Canvas.FillRect(Rect);

  Canvas.MoveTo(0, Height);
  for i := 1 to N do
    Canvas.LineTo(xx(i), Height - m[i]);

  Canvas.TextOut(Width - 140, 10, 'Amplitude ' + IntToStr(Fa));
  Canvas.TextOut(Width - 140, 25, 'Max. freq. ' + IntToStr(Fmf));
  Canvas.TextOut(Width - 140, 40, 'Energy ' + IntToStr(Fte));
  Canvas.TextOut(Width - 140, 55, 'Rate ' + IntToStr(Fc));
end;

procedure TARRockwell.Progress(Done: Longint);
var
  LinData: Pointer;
  LinSize: Longint;
  C, I: Integer;
  Result: Integer;
  TimeOut: Longint;
begin
  if FAnalyzing then
  try
    {
      Step 1. Get next part of modem raw data (already linear, not
      required conveting such as GSM/USR).
    }
    { Fill modem data structure }
    LinData := Pointer(Longint(FAnswerStream.Memory) + FAnswerPos);
    // LinSize := FAnswerStream.Size - FAnswerPos;
    LinSize := FAnswerStream.Position - FAnswerPos;
    // DebugMsgFmt(0, SGotVoiceData, [FAnswerPos, LinSize]);
    { Shift data pointer }
    // FAnswerPos := FAnswerStream.Size;
    FAnswerPos := FAnswerStream.Position;
    {
      Step 2. Analyze voice data (8 bit linear)
    }
    C := LinSize div SizeOf(Byte);
    I := 0;
    while I < C do
    begin
      { Prepare 1024 points of data }
      while (FLinCount < 1024) and (I < C) do
      begin
        FLinBuff^[FLinCount + 1] := TByteArray(LinData^)[I];
        Inc(FLinCount);
        Inc(I);
      end;
      { Check points count }
      if FLinCount >= 1023 then
      begin
        FLinCount := 0;
        { Analyze }
        Result := FFT_AnalyzeEx(FLinBuff, Fa, Fmf, Fte, Fc);

        { **************************************************************
          WARNING! DEBUG ONLY. DRAWING WITHOUT MAIN THREAD SYNCHRONIZING
          MAY CAUSE ERRORS.
          ************************************************************** }

        with SpectrumForm do
        begin
          with PaintBox do DrawSpectrum(BoundsRect, Canvas);
          Update;
        end;

        { Check ringback }
        if Result = ANALYZE_TONE then
        begin
          DebugMsgFmt(0, SAnalyzing, [TimeToStr(Now), Fa, Fmf, Fte, Fc, Result, SToneDetected]);
          { Tone detected, reset voice times }
          FToneTime := Longint(GetTickCount);
          FVoiceTime := 0;
          FWasVoice := False;
        end
        else
        { Voice or silence detected }
        begin
          { Check for required voice }
          if Result = ANALYZE_VOICE then FWasVoice := True;
          { Check voice time }
          if FVoiceTime = 0 then
          begin
            DebugMsgFmt(0, SAnalyzing, [TimeToStr(Now), Fa, Fmf, Fte, Fc, Result, SFirstVoiceDetected]);
            { First voice detection time }
            FVoiceTime:= Longint(GetTickCount);
          end
          else
          begin
            DebugMsgFmt(0, SAnalyzing, [TimeToStr(Now), Fa, Fmf, Fte, Fc, Result, Format(SVoiceDetected, [Abs(Longint(GetTickCount) - FVoiceTime)])]);
            { Get voice detection timeout }
            TimeOut := Profile.RingbackGoesAway * 100;
            if FToneTime = 0 then TimeOut := Profile.RingbackNeverCame * 100;
            { Check voice detection interval }
            if Abs(Longint(GetTickCount) - FVoiceTime) > TimeOut then
              { Check for voice required }
              if (not (Profile.AnswerVoiceRequired)) or FWasVoice then
              begin
                { Voice answer detected }
                FAnswer := VMA_VCON;
                { Stop dialing }
                Modem.StopCurrentAction;
                Exit;
              end;
          end;
        end;
      end;
    end;
  except
    on E: Exception do DebugMsg(L_ERROR, 'Progress: ' + E.Message);
  end;
end;

function TARRockwell.Dial(const Number: string): Integer;
var
  SaveRecMaxLen: Integer;
begin
  { Check modem specified }
  if not CheckModem('Dial', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Process dial with harware answer detection }
  if not Profile.AnswerRecognition then
  begin
    Result := inherited Dial(Number);
    Exit;
  end;
  { Save modifiable parameters }
  SaveRecMaxLen := Profile.RecMaxLen;
  { Dial number }
  if Modem.CommandFmt('ATD%s%s;', '', [Profile.DialMode, Number]) <> S_OK then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Analyze modem answer }
  case Modem.WaitAnswer('', Profile.PortTimeOut * 1000) of
    E_INTERRUPTED:
      begin
        Result := E_INTERRUPTED;
      end;
    VMA_BUSY:
      begin
        Result := S_OK;
        Modem.QueueEvent(TModemEvent.Create(BUSY_TONE));
      end;
    VMA_NO_ANSWER:
      begin
        Result := S_OK;
        Modem.QueueEvent(TModemEvent.Create(NO_ANSWER));
      end;
    VMA_NO_DIAL_TONE:
      begin
        Result := S_OK;
        Modem.QueueEvent(TModemEvent.Create(NO_DIAL_TONE));
      end;
    VMA_OK:
      begin
        { Init FFT }
        if not FFT_Init(1024) then
        begin
          Modem.LastError := VME_FFT_INIT_FAILED;
          Result := S_FAIL;
          Exit;
        end;
        try
          { Init analyze params }
          ANALYZE_FMAX := Profile.AnswerFMax;
          ANALYZE_FTONEMIN := Profile.AnswerFToneMin;
          ANALYZE_FTONEMAX := Profile.AnswerFToneMax;
          ANALYZE_TONELEVEL := Profile.AnswerToneLevel;
          ANALYZE_ENERGYLEVEL := Profile.AnswerEnergyLevel;
          ANALYZE_VOICERATE := Profile.AnswerVoiceRate;
          ANALYZE_NOISERATE := Profile.AnswerNoiseRate;
          { Allocate analyze buffer }
          GetMem(FLinBuff, 1024 * SizeOf(Double));
          try
            FLinCount := 0;
            { Create answer data memory stream }
            FAnswerStream := TMemoryStream.Create;
            try
              { Skip raw data header }
              FAnswerPos := SizeOf(TRMDHeader);
              { Set record timeout }
              Profile.RecMaxLen := Profile.DialTimeOut;
              try
                FConvertError := False;
                FToneTime := 0;
                FVoiceTime := 0;
                FWasVoice := True;
                FAnswer := VMA_NO_ANSWER;
                FTimeOut := False;
                FInterrupted := False;
                { Start recording }
                FAnalyzing := True;
                try
                  if Modem.RecordStream(FAnswerStream) = S_FAIL then
                  begin
                    Result := S_FAIL;
                    Exit;
                  end;
                finally
                  FAnalyzing := False;
                end;
                { Check recognition result }
                if FConvertError then
                begin
                  { Data converting rerror occurred }
                  Modem.LastError := VME_DATA_CONVERT_FAILED;
                  Result := S_FAIL;
                end
                else if FTimeOut then
                begin
                  { Analyzing time expired }
                  Result := S_OK;
                  Modem.QueueEvent(TModemEvent.Create(NO_ANSWER));
                end
                else if FInterrupted then
                begin
                  { Analyzing interrupted }
                  Result := E_INTERRUPTED;
                end
                else if FAnswer = VMA_VCON then
                begin
                  { Voice answer detected }
                  Result := S_OK;
                end
                else if (FToneTime <> 0) or (FVoiceTime <> 0) then
                begin
                  { There are was tones but no voice answer }
                  Result := S_OK;
                  Modem.QueueEvent(TModemEvent.Create(NO_ANSWER));
                end
                else
                begin
                  { Strange result }
                  Modem.LastError := VME_RECOGNITION_FAILED;
                  Result := S_FAIL;
                end;
              finally
                Profile.RecMaxLen := SaveRecMaxLen;
              end;
            finally
              FAnswerStream.Free;
            end;
          finally
            FreeMem(FLinBuff);
          end;
        finally
          FFT_Done;
        end;
      end;
  else
    Modem.LastError := VME_INVALID_MODEM_ANSWER;
    Result := S_FAIL;
  end;
  { Process events queue }
  Modem.CheckEvents;
end;

function TARRockwell.HandleEvent(Event: Integer; Data: TEventData): Integer;
begin
  { Check for modem events }
  case Event of
    NO_VOICE_ENERGY,  { Modem could not detect voice energy on the line }
    SILENCE_DETECTED, { Modem detected silence }
    BUSY_TONE,        { Busy tone detected }
    DIAL_TONE:        { Dial tone detected }
      { Ignore modem events detection while software answer recognition
        in progress }
      if FAnalyzing then
      begin
        Result := S_OK;
        Exit;
      end;
    OPERATION_TIMEOUT:
      { Handle operation timeout event }
      if FAnalyzing then
      begin
        FTimeOut := True;
        Result := S_OK;
        Exit;
      end;
    SIGNAL_STOP:
      { Check for recognition interrrupted }
      if FAnalyzing then FInterrupted := True;
  end;
  { Default handling }
  Result := inherited HandleEvent(Event, Data);
end;

initialization
  ReplaceModemDriver(SRockwellName, TARRockwell);

end.
