{
  Voice Modem Library v1.0 for Delphi 2

  Voice_dll.dll convert library interface.

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit voice_dll;

interface

uses
  Windows;

{
  Usage:

  1. Set required parameters of converting (bits per sample, signed/unsigned ...)
  2. Call SetInputLoacation, SetOutputLocation (i.e. set data source and data
     destination). Default - from file to file.
  3. Call converting function. Any converting function has two parameters:
       - Input file name
       - Output file name.
     Each parmeter is pointer to:
       - string with file name (PChar) if XXXLocation = GETPUT_FILE
       - TInFile structure XXXLocation = GETPUT_MEM

  Note for GSM/USR -> PVF converting:

  At the beginning and end of the data stream a GSM frame (from 38 bytes - a
  headline(2) + data(33) + endline(3)) can be incomplete. In this case 
  incomplete frames are rejected. If the GSM frame is kept 33 bytes (only data),
  it is obliged be full.
}

{ Constants }

const

  { Result codes }

  ERROR_OPEN_FILE = 10;
  ERROR_OPEN_INPUT_FILE = 11;
  ERROR_OPEN_OUTPUT_FILE = 12;
  ERROR_READ_FILE = 13;
  ERROR_READ_RMD_HEADER = 14;
  ERROR_READ_PVF_HEADER = 15;
  ERROR_WRITE_PVF_HEADER = 16;
  ERROR_WRITE_RMD_HEADER = 17;

  ERROR_UNKNOWN_MODEM_TYPE = 20;
  ERROR_UNKNOWN_COMPRESS_METHOD = 21;
  ERROR_UNKNOWN_SPEED = 23;

  ERROR_RMD_TO_PVF = 25;
  ERROR_PVF_TO_RMD = 26;
  ERROR_PVF_TO_WAV = 27;
  ERROR_WAV_TO_PVF = 28;
  ERROR_PVF_TO_LIN = 29;
  ERROR_LIN_TO_PVF = 30;

  ERROR_OUT_OF_MEMORY = 35;
  SUCCESS = 40;

  { Supported modems }

  US_Robotics = 0;
  ZyXEL_1496  = 1;
  ZyXEL_2864  = 2;

  { Data location }

  GETPUT_FILE = 1;  { file }
  GETPUT_MEM  = 2;  { memory }

{ Internal data structures }

type

  TPVFHeader = packed record
    ascii: Integer;
    channels: Integer;
    speed: Integer;
    nbits: Integer;
    read_pvf_data: Pointer;
    write_pvf_data: Pointer;
  end;

  TInFile = packed record
    data: Pointer;
    size: Integer;
    rmd_header: Pointer;
  end;

{ Converting parameters setup functions }

{
  default: false, 8 bits per ample
  allowed: true; 16 or 32
}
procedure SetPvfParameters(ascii: BOOL; nbits: Integer); stdcall;
procedure GetPvfParameters(var ascii: BOOL; var nbits: Integer); stdcall;

{
  default: US_Robotics, 1 (  ,  USR  1)
  allowed: ZyXEL_1496, ZyXEL_2864 with compression 2 or 4
}
procedure SetRmdParameters(mtype, comp: Integer); stdcall;
procedure GetRmdParameters(var mtype, comp: Integer); stdcall;

{
  default: 8
  allowed: 16
}
procedure SetWavParameters(bits: Integer); stdcall;
procedure GetWavParameters(var bits: Integer); stdcall;

{                     
  Parameters: 8/16 bit, sing/unsign, modem specific frequency.
  default: FALSE, FALSE, 8000 (i.e. 8 bit, unsigned, US_Robotics)

  P.S. freq parameters used only for Lin -> Pvf converting
}
procedure SetLinParameters(bits16, is_signed: BOOL; freq: Integer); stdcall;
procedure GetLinParameters(var bits16, is_signed: BOOL; var freq: Integer); stdcall;

{ Data location setup function }

procedure SetInputLocation(_in: Integer);  stdcall;
procedure SetOutputLocation(_out: Integer); stdcall;
function GetInputLocation: Integer; stdcall;
function GetOutputLocation: Integer; stdcall;

{ Converting routines }

function RmdToPvf(name_in, name_out: PChar): Integer; stdcall;
function PvfToRmd(name_in, name_out: PChar): Integer; stdcall;

function PvfToWav(name_in, name_out: PChar): Integer; stdcall;
function WavToPvf(name_in, name_out: PChar): Integer; stdcall;

function PvfToLin(name_in, name_out: PChar): Integer; stdcall;
function LinToPvf(name_in, name_out: PChar): Integer; stdcall;

{ Library locking functions }

function LockVoiceLib(Owner: TObject): Boolean;
function UnlockVoiceLib: Boolean;

function VoiceLibLocked: Boolean;
function VoiceLibLockOwner: TObject;

implementation

const
  voice_dll_name = 'voice_dll.dll';

procedure SetPvfParameters; external voice_dll_name name '__SetPvfParameters@8';
procedure GetPvfParameters; external voice_dll_name name '__GetPvfParameters@8';
procedure SetRmdParameters; external voice_dll_name name '__SetRmdParameters@8';
procedure GetRmdParameters; external voice_dll_name name '__GetRmdParameters@8';
procedure SetWavParameters; external voice_dll_name name '__SetWavParameters@4';
procedure GetWavParameters; external voice_dll_name name '__GetWavParameters@4';
procedure SetLinParameters; external voice_dll_name name '__SetLinParameters@12';
procedure GetLinParameters; external voice_dll_name name '__GetLinParameters@12';
procedure SetInputLocation; external voice_dll_name name '__SetInputLocation@4';
procedure SetOutputLocation; external voice_dll_name name '__SetOutputLocation@4';
function GetInputLocation; external voice_dll_name name '__GetInputLocation@0';
function GetOutputLocation; external voice_dll_name name '__GetOutputLocation@0';
function RmdToPvf; external voice_dll_name name '__RmdToPvf@8';
function PvfToRmd; external voice_dll_name name '__PvfToRmd@8';
function PvfToWav; external voice_dll_name name '__PvfToWav@8';
function WavToPvf; external voice_dll_name name '__WavToPvf@8';
function PvfToLin; external voice_dll_name name '__PvfToLin@8';
function LinToPvf; external voice_dll_name name '__LinToPvf@8';

var
  FLocked: Boolean = False;
  FLockOwner: TObject = nil;
  FLockSection: TRTLCriticalSection;

function LockVoiceLib(Owner: TObject): Boolean;
begin
  EnterCriticalSection(FLockSection);
  { Check for DLL already locked }
  Result := not FLocked;
  { Check lock owner }
  if Result then
  begin
    FLocked := True;
    FLockOwner := Owner;
  end;
  LeaveCriticalSection(FLockSection);
end;

function UnlockVoiceLib: Boolean;
begin
  EnterCriticalSection(FLockSection);
  { Unlocking the DLL }
  Result := FLocked;
  { Check result }
  if Result then
  begin
    FLocked := False;
    FLockOwner := nil;
  end;
  LeaveCriticalSection(FLockSection);
end;

function VoiceLibLocked: Boolean;
begin
  EnterCriticalSection(FLockSection);
  Result := FLocked;
  LeaveCriticalSection(FLockSection);
end;

function VoiceLibLockOwner: TObject;
begin
  EnterCriticalSection(FLockSection);
  Result := FLockOwner;
  LeaveCriticalSection(FLockSection);
end;

initialization
  InitializeCriticalSection(FLockSection);

finalization
  DeleteCriticalSection(FLockSection);

end.
