(****************************************************************************
 *
 *  This file contains functions to compute block alignment of PCM
 *  and IMA ADPCM data.
 *
 *  Based on sample source code for Microsoft IMA ADPCM ACM driver.
 *
 *  Copyright (C) 2000 Roman M. Mochalov
 *  E-mail: roman@sar.nnov.ru
 *
 ****************************************************************************)

unit MMCodec;

{$Q-} 

interface

uses
  Windows, MMSys;

{
  This function compute block alignment of PCM data.

  NOTE! It is _assumed_ that the following fields in the format structure
  are valid:

      nChannels
      wBitsPerSample

  The return value is the block alignment that should be placed in
  the pwfx^.nBlockAlign field.
}

function pcmBlockAlignment(pwfx: PWaveFormatEx): UINT;

{
  This function computes the Samples Per Block of PCM data.

  NOTE! It is _assumed_ that the following fields in the format structure
  are valid:

      nSamplesPerSec
      nBlockAlign

  The return value is the average bytes per second that should be
  placed in the pwfx^.nAvgBytesPerSec field.
}

function pcmAvgBytesPerSec(pwfx: PWaveFormatEx): DWORD;

{ IMA-ADPCM definitions }

const
  IMAADPCM_MAX_CHANNELS    = 2;
  IMAADPCM_BITS_PER_SAMPLE = 4;
  IMAADPCM_WFX_EXTRA_BYTES = 2;  

{
  This function computes the standard block alignment that should
  be used given the TWaveFormatEx structure.

  NOTE! It is _assumed_ that the format is a valid IMA-ADPCM format
  and that the following fields in the format structure are valid:

      nChannels
      nSamplesPerSec

  The return value is the block alignment that should be placed in
  the pwfx^.nBlockAlign field.
}

function imaadpcmBlockAlign(pwfx: PWaveFormatEx): Word;

{
  This function computes the Samples Per Block that should be used
  given the TWaveFormatEx structure.

  NOTE! It is _assumed_ that the format is a valid IMA-ADPCM format
  and that the following fields in the format structure are valid:

      nChannels       = must be 1 or 2!
      nSamplesPerSec
      nBlockAlign

  The return value is the average bytes per second that should be
  placed in the pwfx^.nAvgBytesPerSec field.
}

function imaadpcmSamplesPerBlock(pwfx: PWaveFormatEx): Word;

{
  This function computes the Average Bytes Per Second that should
  be used given the TWaveFormatEx structure.

  NOTE! It is _assumed_ that the format is a valid IMA-ADPCM format
  and that the following fields in the format structure are valid:

     nChannels       = must be 1 or 2!
     nSamplesPerSec
     nBlockAlign

  The return value is the average bytes per second that should be
  placed in the pwfx^.nAvgBytesPerSec field.
}

function imaadpcmAvgBytesPerSec(pwfx: PWaveFormatEx): DWORD;

implementation

function pcmBlockAlignment(pwfx: PWaveFormatEx): UINT;
begin
  Result := (pwfx^.wBitsPerSample shr 3) shl (pwfx^.nChannels shr 1);
end;

function pcmAvgBytesPerSec(pwfx: PWaveFormatEx): DWORD;
begin
  Result := pwfx^.nSamplesPerSec * pwfx^.nBlockAlign;
end;

function imaadpcmBlockAlign(pwfx: PWaveFormatEx): Word;
var
  uBlockAlign: UINT;
  uChannelShift: UINT;
begin
  {
    The data must be an integral number of DWORDs for mono, an even
    number of DWORDs for stereo.
  }
  uChannelShift := pwfx^.nChannels shr 1;
  uBlockAlign := 256 shl uChannelShift;
  {
    Choose a block alignment that makes sense for the sample rate
    that the original PCM data is. Basically, this needs to be
    some reasonable number to allow efficient streaming, etc.
  }
  if pwfx^.nSamplesPerSec > 11025 then
    uBlockAlign := uBlockAlign * (pwfx^.nSamplesPerSec div 11000);
  {
    Return value
  }
  Result := uBlockAlign;
end;

function imaadpcmSamplesPerBlock(pwfx: PWaveFormatEx): Word;
var
  uSamplesPerBlock: UINT;
  uChannelShift: UINT;
  uHeaderBytes: UINT;
  uBitsPerSample: UINT;
begin
  uChannelShift := pwfx^.nChannels shr 1;
  uHeaderBytes := 4 shl uChannelShift;
  uBitsPerSample := IMAADPCM_BITS_PER_SAMPLE shl uChannelShift;
  {
    Compute the 'samples per block' that will be in the encoded
    ADPCM data blocks. This is determined by subtracting out the
    'other info' contained in each block--a block is composed of
    a header followed by the encoded data.

    The block header is composed of the following data:
        2 bytes (16 bit) sample per channel
        1 byte for step table index per channel
        1 byte padding per channel (dword align)

    This gives us (4 * uChannels) bytes of header information that
    contains our first full sample (so we add one below).
  }
  uSamplesPerBlock := (pwfx^.nBlockAlign - uHeaderBytes) * 8;
  uSamplesPerBlock := uSamplesPerBlock div uBitsPerSample;
  uSamplesPerBlock := uSamplesPerBlock + 1;
  {
    Return value
  }
  Result := uSamplesPerBlock;
end;

function imaadpcmAvgBytesPerSec(pwfx: PWaveFormatEx): DWORD;
var
  dwAvgBytesPerSec: DWORD;
  uSamplesPerBlock: UINT;
begin
  uSamplesPerBlock := imaadpcmSamplesPerBlock(pwfx);
  {
    Compute bytes per second including header bytes
  }
  dwAvgBytesPerSec := (pwfx^.nSamplesPerSec * pwfx^.nBlockAlign) div
    uSamplesPerBlock;
  {
    Return value
  }
  Result := dwAvgBytesPerSec;
end;

end.
