{
  Voice Modem Library v1.0

  This file contains specific hardware stuff for Cirrus Logic CL-MD1414XXX
  family voice modems.

  Based on a original source code by Mitch <Mitch.DSouza@NetComm.IE>.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_CLMD1414;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, VML_Modem, VML_Consts, VML_IS101;

const

{ CL-MD1414 available compression modes }

  CLMD1414_CL1     = 1; {?}
  CLMD1414_ADPCM_3 = 3;
  CLMD1414_ADPCM_4 = 4;

{ CL-MD1414 modem strings }

  SCLMD1414Name    = 'Cirrus Logic CL-MD1414';
  SCLMD1414RMDName = 'CL-MD1414';

{$I *.INC}

type

{ TCirrusLogicMD1414 }

  TCirrusLogicMD1414 = class(TIS101)
  public
    constructor Create(AOwner: TComponent); override;
  end;

implementation

{ TCirrusLogicMD1414 }

constructor TCirrusLogicMD1414.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { Modem name }
  ModemName := SCLMD1414Name;
  ModemRMDName := SCLMD1414RMDName;
  { \Q3 - Bidirectional hardware flow control.
    \Q1 - XON/XOFF software flow control. }
  HardFlowCmd := 'AT\Q3';
  SoftFlowCmd := 'AT\Q1';
  { #VLN=1 - Activate VOREL* relay control (i.e. dialup line).
    #VIP=1 - Initialize voice parameter. Relay and speaker driver (AT#VLN=n)
             settings remain unchanged. }
  PickPhoneCmd := 'AT#VLN=1||AT#VIP=1';
  PickPhoneAnsw := 'OK||OK';
  { #VBP - Generate beep tone. }
  BeepCmd := 'AT#VBP';
  { #VPY - Play mode. }
  StartPlayCmd := 'AT#VPY';
  StartPlayAnsw := 'OK|CONNECT';
  IntrPlayCmd := '<DLE>A<DLE><ETX>';
  IntrPlayAnsw := 'OK|VCON';
  StopPlayAnsw := 'OK|VCON';
  { #VRD - Record mode. }
  StartRecCmd := 'AT#VRD';
  StartRecAnsw := 'OK|CONNECT';
  StopRecCmd := '<CR>';
  StopRecAnsw := 'OK|VCON';
  { #VCL=1 - Voice mode.
    #VCL=0 - Data mode. }
  VoiceModeOnCmd := 'AT#VCL=1';
  VoiceModeOffCmd := 'AT#VCL=0';
  { #VSL  - Silence detection threshold level (121-131).
    #VSQT - Record mode 'q' silence timer.
    #VSST - Record and command mode 's' silence timer. }
  SilenceDetectCmd := 'AT#VSL=%d||AT#VSQT=%d||AT#VSST=%:1d';
  SilenceDetectAnsw := 'OK||OK||OK';
  SilenceSensMin := 121;
  SilenceSensMax := 131;
  { #VCSD - Voice command mode silence detection. }
  SilenceDeleteCmd := 'AT#VCSD=%d';
  SilenceDeleteAnsw := 'OK';
  { #VPL - Play mode voice level (121-131). }
  TransmitGainCmd := 'AT#VPL=%d';
  TransmitGainAnsw := 'OK';
  TransmitGainMin := 121;
  TransmitGainMax := 131;
  { #VRL - Recording level (121-131). }
  ReceiveGainCmd := 'AT#VRL=%d';
  ReceiveGainAnsw := 'OK';
  ReceiveGainMin := 121;
  ReceiveGainMax := 131;
  { <DLE>E   - DTMF digit '*'.
    <DLE>F   - DTMF digit '#'.
    <DLE>N   - 1100-Hz fax CNG tone.
    <DLE>e   - Energy detected (dial tone).
    <DLE>g   - 1300-Hz data calling tone (CL-MD1414XX only).
    <DLE>l,L - Remote hang-up. }
  DLECodes.Add('E', RECEIVED_DTMF, '*', '');
  DLECodes.Add('F', RECEIVED_DTMF, '#', '');
  DLECodes.Add('N', FAX_CALLING_TONE, #0, '');
  DLECodes.Add('e', DIAL_TONE, '*', '');
  DLECodes.Add('g', DATA_CALLING_TONE, #0, '');
  DLECodes.Add('l', REMOTE_HANGUP, 'l', '');
  DLECodes.Add('L', REMOTE_HANGUP, 'L', '');
  { #VLN=0  - Disable all.
    #VLN=1  - OHREL* relay control.
    #VLN=2  - VOREL* relay control.
    #VLN=16 - Speaker.
    #VLN=32 - Microphone input. }
  Devices.Clear;
  Devices.Add(NO_DEVICE, SNoDevice, 'AT#VLN=0', 'OK', DF_HIDDEN);
  Devices.Add(DIALUP_LINE, SDialupLine, 'AT#VLN=1', 'OK', DF_TRANSMIT_RECEIVE);
  Devices.Add(EXTERNAL_MICROPHONE, SExternalMicrophone, 'AT#VLN=32','OK', DF_RECEIVE);
  Devices.Add(INTERNAL_SPEAKER, SInternalSpeaker, 'AT#VLN=16','OK', DF_TRANSMIT);
  Devices.Add(LOCAL_HANDSET, SLocalHandset, 'AT#VLS=2','OK', DF_TRANSMIT_RECEIVE);
  { #VSM     - Compression method selection.
    #VSM=CL1 - CL1 9600 Hz (default).
    #VSM=AD3 - ADPCM 3 bits/sample 9600 Hz.
    #VSM=AD4 - ADPCM 4 bits/sample 9600 Hz. }
  Compressions.Clear;
  Compressions.Add(CLMD1414_CL1, SCLMD1414CL1, 'AT#VSR=9600||AT#VSM=CL1', 'OK||OK', 9600, 5, CF_9600 or CF_DEFAULT, 0, 0, 0);
  Compressions.Add(CLMD1414_ADPCM_3, SCLMD1414ADPCM3, 'AT#VSR=9600||AT#VSM=AD3', 'OK||OK', 9600, 3, CF_9600, 0, 0, 0);
  Compressions.Add(CLMD1414_ADPCM_4, SCLMD1414ADPCM4, 'AT#VSR=9600||AT#VSM=AD4', 'OK||OK', 9600, 4, CF_9600, 0, 0, 0);
end;

initialization
  RegisterModemDriver(SCLMD1414Name, TCirrusLogicMD1414);

end.
