{
  Voice Modem Library v1.0

  This file contains generic hardware driver functions for modems that
  follow the IS-101 interim standard for voice modems. Since the commands
  are set in the modem structure, it should be quite generic.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_IS101;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, VML_Modem, VML_Consts, VML_NoModem;

const

{ IS-101 modem strings }

  SIS101Name    = 'IS-101 compatible modem';
  SIS101RMDName = 'IS-101';

{$I *.INC}

type

{ TIS101 }

  TIS101 = class(TNoModem)
  protected
    FStopDialing: Boolean;
    FStopPlaying: Boolean;
    FStopRecording: Boolean;
    FStopWaiting: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    function AnswerPhone: Integer; override;
    function Beep(Frequency, Length: Integer): Integer; override;
    function Dial(const Number: string): Integer; override;
    function EnableCallerID(Enabled: Boolean): Integer; override;
    function EnableDistinctiveRing(Enabled: Boolean): Integer; override;
    function GetVoiceLen(Compression, Speed, Bits: Integer; DataSize: Longint): Integer; override;
    function HandleDLE(Code: Char): Integer; override;
    function HandleEvent(Event: Integer; Data: TEventData): Integer; override;
    function HangUp: Integer; override;
    function Init: Integer; override;
    function InitVoice: Integer; override;
    function PlayDTMF(const Number: string): Integer; override;
    function PlayStreamEx(Stream: TStream; BPS: Integer; DataLength: Longint): Integer; override;
    function RecordStreamEx(Stream: TStream; BPS: Integer; var DataLength: Longint): Integer; override;
    function RestoreVoiceMode: Integer; override;
    function SetCompression(var Compression, Speed, Bits: Integer): Integer; override;
    function SetDevice(Device: Integer): Integer; override;
    function SetReceiveGain(Gain: Integer): Integer; override;
    function SetTransmitGain(Gain: Integer): Integer; override;
    function SetWaveFormat(var WaveFormat, Speed, Bits: Integer): Integer; override;
    function StartPlayStream: Integer; override;
    function StartRecordStream: Integer; override;
    function StopDialing: Integer; override;
    function StopPlayStream: Integer; override;
    function StopPlaying: Integer; override;
    function StopRecordStream: Integer; override;
    function StopRecording: Integer; override;
    function StopWaiting: Integer; override;
    function SwitchToDataFax(const Mode: string): Integer; override;
    function VoiceModeOff: Integer; override;
    function VoiceModeOn: Integer; override;
    function Wait(Timeout: Longint): Integer; override;
    function WaitAnswer(const ExpectedAnswers: string; Timeout: Longint): Integer; override;
  end;

implementation

{ TIS101 }

constructor TIS101.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { Modem name }
  ModemName := SIS101Name;
  ModemRMDName := SIS101RMDName;
  { Flow control }
  HardFlowAnsw := 'OK';
  SoftFlowAnsw := 'OK';
  { H0 - Hang up phone. }
  HangUpCmd := 'ATH0';
  HangUpAnsw := 'OK';
  { +VLS=2 - Dialup line.
    A      - Answer. }
  PickPhoneCmd := 'AT+VLS=2A';
  PickPhoneAnsw := 'OK';
  { X4 - Set extended verbal result messages. }
  ResultCodesCmd := 'ATX4';
  ResultCodesAnsw := 'OK';
  { +VTS=[f,g,n] - DTMF Tone Generation. f and g specify a tone pair, f Hz
                   and g Hz, n = 1-500 }
  BeepCmd := 'AT+VTS=[%d,0,%d]';
  BeepAnsw := 'OK';
  BeepTimeUnit := 10;
  { +VTX - Enter Voice-Transmission mode. }
  StartPlayCmd := 'AT+VTX';
  StartPlayAnsw := 'CONNECT';
  IntrPlayCmd := '<DLE><CAN><DLE><ETX>';
  IntrPlayAnsw := 'OK';
  StopPlayCmd := '<DLE><ETX>';
  StopPlayAnsw := 'OK';
  { +VRX - Start the voice reception process. }
  StartRecCmd := 'AT+VRX';
  StartRecAnsw := 'CONNECT';
  StopRecCmd := '<DLE>!';
  StopRecAnsw := 'OK';
  { +FCLASS   - Switch Data/FAX/Voice mode.
    +FCLASS=0 - Data mode.
    +FCLASS=1 - Fax Class 1.
    +FCLASS=8 - Voice mode. }
  SwitchModeCmd := 'AT+FCLASS=%s';
  SwitchModeAnsw := 'OK';
  VoiceModeOnCmd := 'AT+FCLASS=8';
  VoiceModeOnAnsw := 'OK';
  VoiceModeOffCmd := 'AT+FCLASS=0';
  VoiceModeOffAnsw := 'OK';
  { +VTS=D - Generate default DTMF Tone, default duration. }
  PlayDTMFCmd := 'AT+VTS=%s';
  PlayDTMFExtra := ',%s';
  PlayDTMFAnsw := 'OK';
  { The default baud rate unit is 2400 }
  BaudRateUnit := 2400;
  { The default DTMF length is 1 sec. }
  DTMFLenUnit := 1;
  { <DLE><DLE> - Single <DLE> character in the data stream.
    <DLE><SUB> - Double <DLE> code in data stream.
    <DLE><ETX> - <ETX> code (end of record mode data).
    <DLE>$     - Bong tone detected.
    <DLE>/     - Start of DTMF shielding.
    <DLE>~     - End of DTMF shielding.
    <DLE>0-9,  - DTMF tones 0-9.
    <DLE>*     - DTMF tone *.
    <DLE>#     - DTMF tone #.
    <DLE>A-D   - DTMF tone detected.
    <DLE>a     - Data or fax answer detected (V.25/T.30 2100 Hz).
    <DLE>b,K   - Busy tone detected.
    <DLE>c,m   - Fax calling tone detected (T.30 1100 Hz).
    <DLE>d,i   - Dial tone detected.
    <DLE>e     - European Data Modem Calling Tone (V.25 1300 Hz).
    <DLE>f     - Bell Answer Tone (2225 Hz).
    <DLE>E     - Invalid voice format detected.
    <DLE>h     - Local handset goes on hook.
    <DLE>H     - Local handset goes off hook.
    <DLE>p     - Extension phone goes on hook.
    <DLE>P     - Extension phone goes off hook.
    <DLE>J     - SIT tone detected.
    <DLE>l     - Loop current break.
    <DLE>L     - Loop current polarity reversal.
    <DLE>o     - Receive buffer overrun (oveflow).
    <DLE>q     - Quiet (modem detected silence).
    <DLE>Q     - XON received.
    <DLE>r     - Ringback detected.
    <DLE>R     - Incoming ring detected.
    <DLE>s     - Modem could not detect voice energy on the line.
    <DLE>S     - XOFF received.
    <DLE>t     - TDD detected.
    <DLE>T     - Timing mark will be ignored.
    <DLE>u     - Transmit buffer underrun (becomes empty).
    <DLE>v,V   - Voice detected.
    <DLE>w     - Call waiting, beep interrupt.
    <DLE>Y     - Lost data detected event. }
  DLECodes.Add(DLE, NO_EVENT, #0, SShieldedDLEReceived);
  DLECodes.Add(SUB, NO_EVENT, #0, SShieldedDLEDLEReceived);
  DLECodes.Add(ETX, NO_EVENT, #0, SDLEETXReceived);
  DLECodes.Add('$', BONG_TONE, #0, '');
  DLECodes.Add('/', NO_EVENT, #0, SStartDTMFShielding);
  DLECodes.Add('~', NO_EVENT, #0, SEndDTMFShielding);
  DLECodes.Add('0', RECEIVED_DTMF, '0', '');
  DLECodes.Add('1', RECEIVED_DTMF, '1', '');
  DLECodes.Add('2', RECEIVED_DTMF, '2', '');
  DLECodes.Add('3', RECEIVED_DTMF, '3', '');
  DLECodes.Add('4', RECEIVED_DTMF, '4', '');
  DLECodes.Add('5', RECEIVED_DTMF, '5', '');
  DLECodes.Add('6', RECEIVED_DTMF, '6', '');
  DLECodes.Add('7', RECEIVED_DTMF, '7', '');
  DLECodes.Add('8', RECEIVED_DTMF, '8', '');
  DLECodes.Add('9', RECEIVED_DTMF, '9', '');
  DLECodes.Add('*', RECEIVED_DTMF, '*', '');
  DLECodes.Add('#', RECEIVED_DTMF, '#', '');
  DLECodes.Add('A', RECEIVED_DTMF, 'A', '');
  DLECodes.Add('B', RECEIVED_DTMF, 'B', '');
  DLECodes.Add('C', RECEIVED_DTMF, 'C', '');
  DLECodes.Add('D', RECEIVED_DTMF, 'D', '');
  DLECodes.Add('a', DATA_OR_FAX_DETECTED, #0, '');
  DLECodes.Add('b', BUSY_TONE, 'b', '');
  DLECodes.Add('K', BUSY_TONE, 'K', '');
  DLECodes.Add('c', FAX_CALLING_TONE, 'c', '');
  DLECodes.Add('m', FAX_CALLING_TONE, 'm', '');
  DLECodes.Add('d', DIAL_TONE, 'd', '');
  DLECodes.Add('i', DIAL_TONE, 'i', '');
  DLECodes.Add('e', DATA_CALLING_TONE, 'e', '');
  DLECodes.Add('f', DATA_CALLING_TONE, 'f', '');
  DLECodes.Add('E', NO_EVENT, #0, SInvalidVoiceFormat);
  DLECodes.Add('h', HANDSET_ON_HOOK, 'h', '');
  DLECodes.Add('p', HANDSET_ON_HOOK, 'p', '');
  DLECodes.Add('H', HANDSET_OFF_HOOK, 'H', '');
  DLECodes.Add('P', HANDSET_OFF_HOOK, 'P', '');
  DLECodes.Add('J', SIT_TONE, #0, '');
  DLECodes.Add('l', LOOP_BREAK, #0, '');
  DLECodes.Add('L', LOOP_POLARITY_CHANGE, #0, '');
  DLECodes.Add('o', NO_EVENT, #0, SBufferOverrun);
  DLECodes.Add('q', SILENCE_DETECTED, #0, '');
  DLECodes.Add('Q', NO_EVENT, #0, SXONReceived);
  DLECodes.Add('r', RINGBACK_DETECTED, #0, '');
  DLECodes.Add('R', RING_DETECTED, #0, '');
  DLECodes.Add('s', NO_VOICE_ENERGY, #0, '');
  DLECodes.Add('S', NO_EVENT, #0, SXOFFReceived);
  DLECodes.Add('t', TDD_DETECTED, #0, '');
  DLECodes.Add('T', NO_EVENT, #0, STimingMark);
  DLECodes.Add('u', NO_EVENT, #0, SBufferUnderrun);
  DLECodes.Add('v', VOICE_DETECTED, 'v', '');
  DLECodes.Add('V', VOICE_DETECTED, 'V', '');
  DLECodes.Add('w', CALL_WAITING, #0, '');
  DLECodes.Add('Y', NO_EVENT, #0, SLostData);
  { +VLS=0  - Disconnect all I/O devices. }
  Devices.Add(NO_DEVICE, SNoDevice, 'AT+VLS=0', 'OK', DF_HIDDEN);
end;

function TIS101.AnswerPhone: Integer;
begin
  { Check modem specified }
  if not CheckModem('AnswerPhone', True) then
  begin
    Result := VMA_ERROR;
    Exit;
  end;
  { Answer Phone }
  if Modem.Script(PickPhoneCmd, PickPhoneAnsw, False) <> S_OK then
  begin
    Result := VMA_ERROR;
    Exit;
  end;
  Result := VMA_OK;
end;

function TIS101.Beep(Frequency, Length: Integer): Integer;
var
  Timeout, StartTime: Longint;
begin
  { Check modem specified }
  if not CheckModem('Beep', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Check beep command length }
  if System.Length(BeepCmd) <> 0 then
  begin
    { Write beep command }
    if Modem.CommandFmt(BeepCmd, '', [Frequency, Length div BeepTimeUnit]) <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
    Timeout := Length;
    StartTime := Longint(GetTickCount);
    { Wait modem answer }
    while (Timeout = 0) or (Abs(Longint(GetTickCount) - StartTime) < Timeout) do
    begin
      if Modem.Port.CharReady then Break;
      SleepEx(Modem.Profile.PortReadDelay, False);
    end;
    { Read answer }
    if (Modem.Command('', BeepAnsw) and VMA_USER) <> VMA_USER then
    begin
      Result := S_FAIL;
      Exit;
    end;
  end;
  { Success }
  Result := S_OK;
end;

function TIS101.Dial(const Number: string): Integer;
var
  Command, Answer: string;
  Timeout, StartTime: Longint;
  DialComplete, WasTimeout: Boolean;
begin
  { Check modem specified }
  if not CheckModem('Dial', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Setup dialing }
  if Length(DialSetupCmd) <> 0 then
    if Modem.Script(DialSetupCmd, DialSetupAnsw, False) <> S_OK then
    begin
      DebugMsg(L_ERROR, SDialSetupError);
      Result := S_FAIL;
      Exit;
    end;
  FStopDialing := False;
  { Format dial command }
  Command := Format('ATD%s%s', [Profile.DialMode, Number]);
  { Dial number }
  if Modem.Command(Command, '') <> S_OK then
  begin
    Result := S_FAIL;
    Exit;
  end;
  Timeout := Profile.DialTimeout * 1000;
  StartTime := Longint(GetTickCount);
  DialComplete := False;
  WasTimeout := False;
  Result := S_FAIL;
  { Wait modem answer }
  while (not FStopDialing) and (not DialComplete) do
  begin
    { Check for modem char }
    if Modem.Port.CharReady then
    begin
      { Read modem answer string }
      if Modem.ReadStr(Answer) <> S_OK then
      begin
        Result := S_FAIL;
        Exit;
      end;
      { Convert answer string }
      Result := Modem.Analyze(Answer, Command);
      { Analyze answer }
      case Result of
        VMA_BUSY:
          begin
            DialComplete := True;
            Result := S_OK;
            Modem.QueueEvent(TModemEvent.Create(BUSY_TONE));
          end;
        VMA_EMPTY:
          begin
          end;
        VMA_NO_ANSWER:
          begin
            DialComplete := True;
            Result := S_OK;
            Modem.QueueEvent(TModemEvent.Create(NO_ANSWER));
          end;
        VMA_NO_DIAL_TONE:
          begin
            DialComplete := True;
            Result := S_OK;
            Modem.QueueEvent(TModemEvent.Create(NO_DIAL_TONE));
          end;
        VMA_RINGING:
          begin
            Modem.QueueEvent(TModemEvent.Create(RINGING_TONE));
          end;
        VMA_USER_1:
          begin
          end;
        VMA_NO_CARRIER:
          begin
            DialComplete := True;
            Result := S_OK;
            Modem.QueueEvent(TModemEvent.Create(NO_CARRIER));
          end;
        VMA_CONNECT:
          begin
            DialComplete := True;
            Result := S_OK;
            Modem.QueueEvent(TModemEvent.Create(CONNECT_DETECTED));
          end;
        VMA_OK, VMA_VCON:
          begin
            DialComplete := True;
            Result := S_OK;
          end;
      else
        Modem.LastError := VME_INVALID_MODEM_ANSWER;
        DialComplete := True;
        Result := S_FAIL;
      end;
    end
    else
      { Wait for data }
      SleepEx(Profile.PortReadDelay, False);
    { Yield callback }
    YieldProc;
    { Check for new events }
    Modem.CheckEvents;
    { Check timeout }
    if (Timeout > 0) and (Abs(Longint(GetTickCount) - StartTime) >= Timeout) and
      (not FStopDialing) and (not DialComplete) then
    begin
      WasTimeout := True;
      Modem.QueueEvent(TModemEvent.Create(OPERATION_TIMEOUT));
      Modem.StopDialing;
    end;
  end;
  { Check for dialing interrupted }
  if WasTimeout then Result := E_TIMEOUT
  else if FStopDialing then Result := E_INTERRUPTED;
  { Process events queue }
  Modem.CheckEvents;
end;

function TIS101.EnableCallerID(Enabled: Boolean): Integer;
var
  CallerIDCmd, CallerIDAnsw: string;
begin
  { Check modem specified }
  if not CheckModem('EnableCallerID', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Enable Caller ID }
  if Length(CallerIDCmd) <> 0 then
  begin
    { Get command }
    if Enabled then
    begin
      CallerIDCmd := CallerIDOnCmd;
      CallerIDAnsw := CallerIDOnAnsw;
    end
    else
    begin
      CallerIDCmd := CallerIDOffCmd;
      CallerIDAnsw := CallerIDOffAnsw;
    end;
    { Execute command }
    if Modem.Script(CallerIDCmd, CallerIDAnsw, False) <> S_OK then
    begin
      DebugMsg(L_WARN, SCallerIDError);
      Result := S_FAIL;
      Exit;
    end;
  end;
  { Success }
  Result := S_OK;
end;

function TIS101.EnableDistinctiveRing(Enabled: Boolean): Integer;
var
  DRingCmd, DRingAnsw: string;
begin
  { Check modem specified }
  if not CheckModem('EnableDistinctiveRing', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Enable distinctive ring }
  if Length(DRingCmd) <> 0 then
  begin
    { Get command }
    if Enabled then
    begin
      DRingCmd := DRingOnCmd;
      DRingAnsw := DRingOnAnsw;
    end
    else
    begin
      DRingCmd := DRingOffCmd;
      DRingAnsw := DRingOffAnsw;
    end;
    { Execute command }
    if Modem.Script(DRingCmd, DRingAnsw, False) <> S_OK then
    begin
      DebugMsg(L_WARN, SDistinctiveRingError);
      Result := S_FAIL;
      Exit;
    end;
  end;
  { Success }
  Result := S_OK;
end;

function TIS101.GetVoiceLen(Compression, Speed, Bits: Integer; DataSize: Longint): Integer;
var
  BPS: Integer;
begin
  BPS := (Speed * Bits) div 8;
  { Check BPS and return duration of voice message }
  if BPS > 0 then
  begin
    Result := Round(DataSize / BPS * 1000);
    Exit;
  end;
  { Error }
  Result := 0;
end;

function TIS101.HandleDLE(Code: Char): Integer;
var
  DLECode: TModemDLECode;
  Event: TModemEvent;
begin
  { Check modem specified }
  if not CheckModem('HandleDLE', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Handle DLE code }
  if DLECodes.Find(Code, DLECode) then
  begin
    { Debug message }
    if Length(DLECode.DebugMsg) <> 0 then DebugMsg(L_WARN, DLECode.DebugMsg);
    { Queue modem event }
    if DLECode.Event <> NO_EVENT then
    begin
      Event := TModemEvent.CreateDataC(DLECode.Event, DLECode.EventCode);
      Result := Modem.QueueEvent(Event);
      { Check form DTMF code }
      if DLECode.Event = RECEIVED_DTMF then
        Modem.LastDTMF := Modem.LastDTMF + DLECode.EventCode;
    end
    else
      Result := S_OK;
    { Code handled }
    Exit;
  end;
  { Unknown DLE code }
  DebugMsgFmt(L_WARN, SUnknownDLECode, [Code]);
  Result := S_FAIL;
end;

function TIS101.HandleEvent(Event: Integer; Data: TEventData): Integer;
begin
  Result := UNKNOWN_EVENT;
end;

function TIS101.HangUp: Integer;
begin
  { Check modem specified }
  if not CheckModem('HangUp', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Hang up }
  if Length(HangUpCmd) <> 0 then
    if Modem.Script(HangUpCmd, HangUpAnsw, False) <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
  Result := S_OK;
end;

function TIS101.Init: Integer;
begin
  DebugMsgFmt(L_MESG, SInitModem, [ModemRMDName]);
  { Check modem specified }
  if not CheckModem('Init', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Turn on hardware flow control }
  if Length(HardFlowCmd) <> 0 then
    if (Modem.Command(HardFlowCmd, HardFlowAnsw) and VMA_USER) <> VMA_USER then
    begin
      DebugMsg(L_WARN, SFlowControlError);
      Result := S_FAIL;
      Exit;
    end;
  { Set extended verbal result messages }
  if Length(ResultCodesCmd) <> 0 then
    if (Modem.Command(ResultCodesCmd, ResultCodesAnsw) and VMA_USER) <> VMA_USER then
    begin
      DebugMsg(L_WARN, SResultCodesError);
      Result := S_FAIL;
      Exit;
    end;
  { Device-specific initialization command }
  if Length(DevInitCmd) <> 0 then
    if Modem.Script(DevInitCmd, DevInitAnsw, True) <> S_OK then
      DebugMsg(L_WARN, SInitCmdFailed);
  { Init complete }
  Result := S_OK;
end;

function TIS101.InitVoice: Integer;
var
  BaudRate: Integer;
  SilenceSens: Integer;
  DTMFSens, DTMFLen: Integer;
begin
  DebugMsgFmt(L_MESG, SInitVoice, [ModemRMDName]);
  { Check modem specified }
  if not CheckModem('InitVoice', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Select required modem/DTE baud rate }
  if Length(BaudRateCmd) <> 0 then
  begin
    BaudRate := Profile.VoiceBaudRate div BaudRateUnit;
    if Modem.ScriptFmt(BaudRateCmd, BaudRateAnsw, False, [BaudRate]) <> S_OK then
      DebugMsg(L_WARN, SBaudRateError);
  end;
  { Silence detection }
  if Length(SilenceDetectCmd) <> 0 then
  begin
    SilenceSens := SilenceSensMin + Profile.RecSilenceSens * (SilenceSensMax - SilenceSensMin) div 100;
    if Modem.ScriptFmt(SilenceDetectCmd, SilenceDetectAnsw, False, [SilenceSens, Profile.RecSilenceLen]) <> S_OK then
      DebugMsg(L_WARN, SSilenceDetectionError);
  end;
  { Silence deletion }
  if Length(SilenceDeleteCmd) <> 0 then
    if Modem.ScriptFmt(SilenceDeleteCmd, SilenceDeleteAnsw, False, [Ord(Profile.RecRemoveSilence)]) <> S_OK then
      DebugMsg(L_WARN, SSilenceDeletionError);
  { Delay after ringback or before any ringback before modem assumes phone
    has been answered }
  if Length(RingbackTimersCmd) <> 0 then
    if Modem.ScriptFmt(RingbackTimersCmd, RingbackTimersAnsw, False, [Profile.RingbackGoesAway, Profile.RingbackNeverCame]) <> S_OK then
      DebugMsg(L_WARN, SRingbackTimersError);
  { DTMF tones detection }
  if Length(DTMFDetectCmd) <> 0 then
  begin
    DTMFSens := DTMFSensMin + Profile.DTMFSens * (DTMFSensMax - DTMFSensMin) div 100;
    DTMFLen := Profile.DTMFLen div DTMFLenUnit;
    if Modem.ScriptFmt(DTMFDetectCmd, DTMFDetectAnsw, False, [DTMFSens, DTMFLen]) <> S_OK then
      DebugMsg(L_WARN, SDTMFDetectionError);
  end;
  { Set transmit gain (speaker volume) }
  SetTransmitGain(Profile.TransmitGain);
  { Set receive gain (sensitive of the microphone) }
  SetReceiveGain(Profile.ReceiveGain);
  { Caller identification }
  EnableCallerID(Profile.CallerIDDetection);
  { Distinctive ringing reporting }
  EnableDistinctiveRing(Profile.DistinctiveRing);
  { Device-specific voice initialization command }
  if Length(DevInitVoiceCmd) <> 0 then
    if Modem.Script(DevInitVoiceCmd, DevInitVoiceAnsw, True) <> S_OK then
      DebugMsg(L_WARN, SInitVoiceCmdFailed);
  { Init voice complete }
  Result := S_OK;
end;

function TIS101.PlayDTMF(const Number: string): Integer;
var
  I, L: Integer;
  Command: string;
begin
  { Check modem specified }
  if not CheckModem('PlayDTMF', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Check command }
  if Length(PlayDTMFCmd) <> 0 then
  begin
    L := Length(Number);
    { Check number }
    if L = 0 then
    begin
      Result := S_FAIL;
      Exit;
    end;
    { Format command }
    try
      Command := Format(PlayDTMFCmd, [Number[1]]);
      for I := 2 to L do Command := Format(Command + PlayDTMFExtra, [Number[I]]);
    except
      DebugMsg(L_ERROR, SInvalidDTMFCommandFormat);
      Modem.LastError := VME_INVALID_COMMAND_FORMAT;
      Result := S_FAIL;
      Exit;
    end;
    { Send command }
    if Modem.Command(Command, '') <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
    { Delay }
    SleepEx(L * 500, False);
    { Read modem answer }
    if (Modem.Command('', PlayDTMFAnsw) and VMA_USER) <> VMA_USER then
    begin
      Result := S_FAIL;
      Exit;
    end;
  end;
  { Success }
  Result := S_OK;
end;

function TIS101.PlayStreamEx(Stream: TStream; BPS: Integer; DataLength: Longint): Integer;
var
  ReadBuffer: array[0..VOICE_BUF_LEN - 1] of Char;
  ReadLen, ReadPos, ReadMax: Integer;
  OutputBuffer: array[0..VOICE_BUF_LEN + 1] of Char;
  BytesMax, BytesOut: Integer;
  DataByte, ModemByte: Char;
  PlayComplete, WasTimeout: Boolean;
  Timeout, StartTime: Longint;

  function ReadDataByte(var C: Char): Integer;
  var
    L: Longint;
  begin
    { Read stream buffer }
    if ReadPos = ReadMax then
    begin
      L := VOICE_BUF_LEN;
      if L > ReadLen then L := ReadLen;
      { Read buffer }
      ReadMax := Stream.Read(ReadBuffer, L);
      ReadPos := 0;
      Dec(ReadLen, ReadMax);
      { Check EOF }
      if ReadMax = 0 then
      begin
        Result := 0;
        Exit;
      end;
    end;
    { Get data byte }
    C := ReadBuffer[ReadPos];
    Inc(ReadPos);
    { Success }
    Result := 1;
  end;

begin
  { Check modem specified }
  if not CheckModem('PlayStream', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  ReadLen := DataLength;
  if ReadLen = 0 then ReadLen := Stream.Size - Stream.Position;
  ReadPos := 0;
  ReadMax := 0;
  BytesMax := (BPS div 8) div 10;
  if BytesMax > VOICE_BUF_LEN then BytesMax := VOICE_BUF_LEN;
  PlayComplete := False;
  if Profile.PlayPreview then
    Timeout := Profile.PlayPreviewLen * 1000
  else
    Timeout := Profile.PlayMaxLen * 1000;
  StartTime := GetTickCount;
  WasTimeout := False;
  { Playing voice data }
  while (not FStopPlaying) and (not PlayComplete) do
  begin
    { Read voice data from stream and prepare to output }
    BytesOut := 0;
    while BytesOut < BytesMax do
    begin
      { Read byte from stream }
      if ReadDataByte(DataByte) = 0 then
      begin
        PlayComplete := True;
        Break;
      end;
      { Add byte to output buffer }
      OutputBuffer[BytesOut] := DataByte;
      Inc(BytesOut);
      { Check for shielded <DLE> codes }
      if DataByte = DLE then
      begin
        OutputBuffer[BytesOut] := DLE;
        Inc(BytesOut);
      end;
      { Yield callback }
      if BytesOut mod 16 = 0 then YieldProc;
    end;
    { Send buffer to modem }
    if BytesOut > 0 then
    begin
      DebugMsgFmt(L_JUNK, SVoiceData, [BytesOut]);
      { Send data }
      if Modem.Write(@OutputBuffer, BytesOut) = S_FAIL then
      begin
        Result := S_FAIL;
        Exit;
      end;
      { Progress callback }
      Progress(Round(Stream.Position / Stream.Size * 100));
    end;
    { Check for modem char }
    while Modem.Port.CharReady do
    begin
      if Modem.ReadChar(ModemByte) = S_FAIL then
      begin
        Result := S_FAIL;
        Exit;
      end;
      { Check for <DLE> codes }
      if ModemByte = DLE then
      begin
        if Modem.ReadChar(ModemByte) = S_FAIL then
        begin
          Result := S_FAIL;
          Exit;
        end;
        DebugMsgFmt(L_JUNK, SReadDLE, [ModemByte]);
        Modem.HandleDLE(ModemByte);
      end
      else
        DebugMsgFmt(L_WARN, SUnexpectedByte, [ModemByte]);
    end;
    { Check for new events }
    Modem.CheckEvents;
    { Check timeout }
    if ((TimeOut > 0) and (Abs(Longint(GetTickCount) - StartTime) >= Timeout)) and (not FStopPlaying) then
    begin
      WasTimeout := True;
      Modem.QueueEvent(TModemEvent.Create(OPERATION_TIMEOUT));
      Modem.StopPlaying;
    end;
  end;
  { Check for playing stopped }
  if WasTimeout then Result := E_TIMEOUT
  else if FStopPlaying then Result := E_INTERRUPTED
  else Result := S_OK;
  { Process events queue }
  Modem.CheckEvents;
end;

function TIS101.RecordStreamEx(Stream: TStream; BPS: Integer; var DataLength: Longint): Integer;
var
  InputBuffer: array[0..VOICE_BUF_LEN + 1] of Char;
  InputByte: Char;
  BytesMax, BytesIn: Integer;
  GotDLEETX, WasDLE: Boolean;
  Timeout, StartTime: Longint;
  WasTimeout: Boolean;

  function WriteInputBuffer: Boolean;
  begin
    Result := True;
    if Stream.Write(InputBuffer, BytesIn) <> BytesIn then
    begin
      DebugMsgFmt(L_ERROR, SDataWriteError, [GetLastError]);
      Modem.LastError := VME_STREAM_WRITE_ERR;
      Result := False;
    end;
  end;

begin
  { Check modem specified }
  if not CheckModem('RecordStream', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  GotDLEETX := False;
  WasDLE := False;
  BytesMax := (BPS div 8) div 10;
  if BytesMax > VOICE_BUF_LEN then BytesMax := VOICE_BUF_LEN;
  BytesIn := 0;
  DataLength := 0;
  Timeout := Profile.RecMaxLen * 1000;
  StartTime := Longint(GetTickCount);
  WasTimeout := False;
  { Recording voice data }
  DebugMsg(L_JUNK, SStartRecording);
  while not GotDLEETX do
  begin
    { Check timeout }
    if (TimeOut > 0) and (Abs(Longint(GetTickCount) - StartTime) >= Timeout) and (not FStopRecording) then
    begin
      WasTimeout := True;
      Modem.QueueEvent(TModemEvent.Create(OPERATION_TIMEOUT));
      Modem.StopRecording;
    end;
    { Receive data (check error if modem has no data during timeout) }
    if Modem.Read(@InputByte, 1) <> 1 then
    begin
      Result := S_FAIL;
      Exit;
    end;
    { Check about DLE code }
    if WasDLE then
    begin
      WasDLE := False;
      { Analyse <DLE> char }
      case InputByte of
        DLE:
          { Single <DLE> code }
          begin
            InputBuffer[BytesIn] := DLE;
            Inc(BytesIn);
          end;
        SUB:
          { Double <DLE> code }
          begin
            InputBuffer[BytesIn] := DLE;
            Inc(BytesIn);
            InputBuffer[BytesIn] := DLE;
            Inc(BytesIn);
          end;
        ETX:
          { End of Stream }
          begin
            DebugMsgFmt(L_JUNK, SVoiceData, [BytesIn]);
            DebugMsg(L_JUNK, SDLEETX);
            { Record complete }
            GotDLEETX := True;
            { Flush buffer }
            if not WriteInputBuffer then
            begin
              Result := E_INTERRUPTED;
              Exit;
            end;
            Inc(DataLength, BytesIn);
            { Handle DLE code }
            Modem.HandleDLE(InputByte);
            { Progress callback }
            Progress(Stream.Size);
          end;
      else
        { DLE code }
        DebugMsgFmt(L_JUNK, SReadDLE, [InputByte]);
        Modem.HandleDLE(InputByte);
      end;
    end
    else
      { Check input byte }
      if InputByte = DLE then
        { DLE byte received }
        WasDLE := True
      else
      begin
        { Data byte received }
        InputBuffer[BytesIn] := InputByte;
        Inc(BytesIn);
      end;
    { Yield callback }
    if BytesIn mod 16 = 0 then YieldProc;
    { Write buffer to stream }
    if (not GotDLEETX) and (BytesIn >= BytesMax) then
    begin
      DebugMsgFmt(L_JUNK, SVoiceData, [BytesIn]);
      { Write data }
      if not WriteInputBuffer then
      begin
        Result := E_INTERRUPTED;
        Exit;
      end;
      Inc(DataLength, BytesIn);
      BytesIn := 0;
      { Progress callback }
      Progress(Stream.Size);
    end;
    { Check for new events }
    Modem.CheckEvents;
  end;
  { Process events queue }
  Modem.CheckEvents;
  { Recording complete (always interrupted or timeout) }
  if WasTimeout then Result := E_TIMEOUT
  else Result := E_INTERRUPTED;
end;

function TIS101.RestoreVoiceMode: Integer;
var
  BaudRate: Integer;
begin
  { Entering voice mode }
  if VoiceModeOn = S_FAIL then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Restore required modem/DTE baud rate }
  if Length(BaudRateCmd) <> 0 then
  begin
    BaudRate := Profile.VoiceBaudRate div BaudRateUnit;
    if Modem.ScriptFmt(BaudRateCmd, BaudRateAnsw, False, [BaudRate]) <> S_OK then
      DebugMsg(L_WARN, SBaudRateError);
  end;
  { Additional restoring voice command }
  if Length(RestoreVoiceCmd) <> 0 then
    if Modem.Script(RestoreVoiceCmd, RestoreVoiceAnsw, True) <> S_OK then
      DebugMsg(L_WARN, SRestoreVoiceCmdFailed);
  { Success }
  Result := S_OK;
end;

function TIS101.SetCompression(var Compression, Speed, Bits: Integer): Integer;
var
  ModemCompression: TModemCompression;
  Flag: Integer;
begin
  { Check modem specified }
  if not CheckModem('SetCompression', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set compression }
  if Compressions.Count > 0 then
  begin
    { Find compression }
    if not Compressions.Find(Compression, ModemCompression) then
    begin
      { Illegal compression method }
      DebugMsgFmt(L_WARN, SIllegalCompressionMethod, [ModemRMDName, Compression]);
      Modem.LastError := VME_ILLEGAL_COMPRESSION;
      Result := S_FAIL;
      Exit;
    end;
    { Check compression sampling rate }
    case Speed of
      0: Flag := 0;
      7200: Flag := CF_7200;
      8000: Flag := CF_8000;
      9600: Flag := CF_9600;
      11025: Flag := CF_11025;
    else
      Flag := -1;
    end;
    if (Flag = -1) or ((Flag <> 0) and (ModemCompression.Flags and Flag = 0)) then
    begin
      DebugMsgFmt(L_WARN, SIllegalSampleRate, [ModemRMDName, Speed]);
      Modem.LastError := VME_ILLEGAL_SAMPLE_RATE;
      Result := S_FAIL;
      Exit;
    end;
    { Compression params }
    Compression := ModemCompression.Compression;
    if Speed = 0 then Speed := ModemCompression.Speed;
    Bits := ModemCompression.Bits;
    { Compression setup command }
    if Modem.ScriptFmt(ModemCompression.Command, ModemCompression.Answer, True, [Speed]) <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
  end;
  { Success }
  Result := S_OK;
end;

function TIS101.SetDevice(Device: Integer): Integer;
var
  ModemDevice: TModemDevice;
begin
  { Check modem specified }
  if not CheckModem('SetDevice', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set device }
  if Devices.Count > 0 then
  begin
    { Find device }
    if not Devices.Find(Device, ModemDevice) then
    begin
      { Given device unknown }
      DebugMsgFmt(L_WARN, SUnknownDevice, [ModemRMDName, Device]);
      Modem.LastError := VME_UNKNOWN_DEVICE;
      Result := S_FAIL;
      Exit;
    end;
    { Device setup command }
    if Modem.Script(ModemDevice.Command, ModemDevice.Answer, True) <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
  end;
  { Success }
  Result := S_OK;
end;

function TIS101.SetReceiveGain(Gain: Integer): Integer;
begin
  { Check modem specified }
  if not CheckModem('SetReceiveGain', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set receive gain }
  if Length(ReceiveGainCmd) <> 0 then
  begin
    { Translate gain valud to the modem units }
    if Gain = -1 then Gain := 75;
    Gain := ReceiveGainMin + Gain * (ReceiveGainMax - ReceiveGainMin) div 100;
    { Execute setup command }
    if Modem.ScriptFmt(ReceiveGainCmd, ReceiveGainAnsw, False, [Gain]) <> S_OK then
    begin
      DebugMsg(L_WARN, SReceiveGainError);
      Result := S_FAIL;
      Exit;
    end;
  end;
  { Success }
  Result := S_OK;
end;

function TIS101.SetTransmitGain(Gain: Integer): Integer;
begin
  { Check modem specified }
  if not CheckModem('SetTransmitGain', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set transmit gain }
  if Length(TransmitGainCmd) <> 0 then
  begin
    { Translate gain value to the modem units }
    if Gain = -1 then Gain := 75;
    Gain := TransmitGainMin + Gain * (TransmitGainMax - TransmitGainMin) div 100;
    { Execute setup command }
    if Modem.ScriptFmt(TransmitGainCmd, TransmitGainAnsw, False, [Gain]) <> S_OK then
    begin
      DebugMsg(L_WARN, STransmitGainError);
      Result := S_FAIL;
      Exit;
    end;
  end;
  { Success }
  Result := S_OK;
end;

function TIS101.SetWaveFormat(var WaveFormat, Speed, Bits: Integer): Integer;
var
  ModemCompression: TModemCompression;
  WaveFlag: Integer;
begin
  { Check modem specified }
  if not CheckModem('SetWaveFormat', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set compression for given wave format }
  if Compressions.Count > 0 then
  begin
    { Find compression }
    if not Compressions.FindWave(WaveFormat, ModemCompression) then
    begin
      { Illegal format }
      DebugMsgFmt(L_WARN, SIllegalWaveFormat, [ModemRMDName, WaveFormat]);
      Modem.LastError := VME_ILLEGAL_WAVE_FORMAT;
      Result := S_FAIL;
      Exit;
    end;
    { Check sampling rate }
    case Speed of
      0: WaveFlag := 0;
      7200: WaveFlag := CF_7200;
      8000: WaveFlag := CF_8000;
      9600: WaveFlag := CF_9600;
      11025: WaveFlag := CF_11025;
    else
      WaveFlag := -1;
    end;
    if (WaveFlag = -1) or ((WaveFlag <> 0) and (ModemCompression.WaveFlags and WaveFlag = 0)) then
    begin
      DebugMsgFmt(L_WARN, SIllegalSampleRate, [ModemRMDName, Speed]);
      Modem.LastError := VME_ILLEGAL_SAMPLE_RATE;
      Result := S_FAIL;
      Exit;
    end;
    { Format compression params }
    WaveFormat := ModemCompression.WaveFormat;
    if Speed = 0 then Speed := ModemCompression.WaveSpeed;
    Bits := ModemCompression.Bits;
    { Compression setup command }
    if Modem.ScriptFmt(ModemCompression.Command, ModemCompression.Answer, True, [Speed]) <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
  end;
  { Success }
  Result := S_OK;
end;

function TIS101.StartPlayStream: Integer;
begin
  FStopPlaying := False;
  { Check modem specified }
  if not CheckModem('StartPlayStream', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set the DCE's initial value for the DTE/DCE inactivity timer }
  if Length(InactivityTimerCmd) <> 0 then
    if (Modem.CommandFmt(InactivityTimerCmd, InactivityTimerAnsw, [Profile.InactivityTimeout]) and VMA_USER) <> VMA_USER then
      DebugMsg(L_ERROR, SInactivityTimerOnError);
  { Start playing }
  if Modem.Script(StartPlayCmd, StartPlayAnsw, False) <> S_OK then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { <START PLAYING> }
  DebugMsg(L_JUNK, SStartPlaying);
  { Success }
  Result := S_OK;
end;

function TIS101.StartRecordStream: Integer;
begin
  FStopRecording := False;
  { Check modem specified }
  if not CheckModem('StartRecordStream', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set the DCE's initial value for the DTE/DCE inactivity timer }
  if Length(InactivityTimerCmd) <> 0 then
    if (Modem.CommandFmt(InactivityTimerCmd, InactivityTimerAnsw, [Profile.InactivityTimeout]) and VMA_USER) <> VMA_USER then
      DebugMsg(L_ERROR, SInactivityTimerOnError);
  { Tells the modem ready to receive }
  if Modem.Script(StartRecCmd, StartRecAnsw, False) <> S_OK then
  begin
    Result := S_FAIL;
    Exit;
  end;
  Result := S_OK;
end;

function TIS101.StopDialing: Integer;
begin
  FStopDialing := True;
  Result := S_OK;
end;

function TIS101.StopPlayStream: Integer;
const
  NULByte: Char = NUL;
begin
  { Check modem specified }
  if not CheckModem('StopPlayStream', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Check for play already interrupted }
  if FStopPlaying then
  begin
    { <INTERRUPT PLAYING> }
    DebugMsg(L_JUNK, SInterruptPlaying);
    { Flush output buffer }
    Modem.Port.FlushOutBuffer;
    { Write null char }
    if Modem.Write(@NULByte, 1) <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
    { Write interrupt recording command }
    if Modem.WriteCmd(IntrPlayCmd) <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
    { Wait modem answer it is done }
    if (Modem.Command('', IntrPlayAnsw) and VMA_USER) <> VMA_USER then
    begin
      Result := S_FAIL;
      Exit;
    end;
  end
  else
  begin
    { <STOP PLAYING> }
    DebugMsg(L_JUNK, SStopPlaying);
    { Write stop recording command }
    if Modem.WriteCmd(StopPlayCmd) <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
    { Wait modem answer it is done }
    if (Modem.Command('', StopPlayAnsw) and VMA_USER) <> VMA_USER then
    begin
      Result := S_FAIL;
      Exit;
    end;
  end;
  { Turn off DTE/DCE inactivity timer }
  if Length(InactivityTimerCmd) <> 0 then
    if (Modem.CommandFmt(InactivityTimerCmd, InactivityTimerAnsw, [0]) and VMA_USER) <> VMA_USER then
      DebugMsg(L_ERROR, SInactivityTimerOffError);
  { Playing complete }
  Result := S_OK;
end;

function TIS101.StopPlaying: Integer;
begin
  FStopPlaying := True;
  Result := S_OK;
end;

function TIS101.StopRecordStream: Integer;
begin
  { Check modem specified }
  if not CheckModem('StopRecordStream', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Wait modem answer it is done }
  if (Modem.Command('', StopRecAnsw) and VMA_USER) <> VMA_USER then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Turn off DTE/DCE inactivity timer }
  if Length(InactivityTimerCmd) <> 0 then
    if (Modem.CommandFmt(InactivityTimerCmd, InactivityTimerAnsw, [0]) and VMA_USER) <> VMA_USER then
      DebugMsg(L_ERROR, SInactivityTimerOffError);
  { Recording complete }
  Result := S_OK;
end;

function TIS101.StopRecording: Integer;
begin
  FStopRecording := True;
  { Check modem specified }
  if not CheckModem('StopRecording', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  DebugMsg(L_JUNK, SStopRecording);
  { Write stop recording sequence }
  if Modem.WriteCmd(StopRecCmd) <> S_OK then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Success }
  Result := S_OK;
end;

function TIS101.StopWaiting: Integer;
begin
  FStopWaiting := True;
  Result := S_OK;
end;

function TIS101.SwitchToDataFax(const Mode: string): Integer;
begin
  { Check modem specified }
  if not CheckModem('SwitchToDataFax', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Write switch command }
  if Length(SwitchModeCmd) <> 0 then
    if Modem.ScriptFmt(SwitchModeCmd, SwitchModeAnsw, False, [Mode]) <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
  { Check modem }
  if Modem.Command('AT', 'OK') <> VMA_USER_1 then
  begin
    Result := S_FAIL;
    Exit;
  end;
  Result := S_OK;
end;

function TIS101.VoiceModeOff: Integer;
begin
  { Check modem specified }
  if not CheckModem('VoiceModeOff', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Write voice mode off command }
  if Length(VoiceModeOffCmd) <> 0 then
    if Modem.Script(VoiceModeOffCmd, VoiceModeOffAnsw, False) <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
  Result := S_OK;
end;

function TIS101.VoiceModeOn: Integer;
begin
  { Check modem specified }
  if not CheckModem('VoiceModeOn', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Write voice mode on command }
  if Length(VoiceModeOnCmd) <> 0 then
    if Modem.Script(VoiceModeOnCmd, VoiceModeOnAnsw, False) <> S_OK then
    begin
      Result := S_FAIL;
      Exit;
    end;
  Result := S_OK;
end;

function TIS101.Wait(Timeout: Longint): Integer;
var
  CharRead: Char;
  StartTime: Longint;
  WasTimeout: Boolean;
begin
  { Check modem specified }
  if not CheckModem('Wait', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  FStopWaiting := False;
  StartTime := GetTickCount;
  WasTimeout := False;
  { Wait modem answer }
  while not FStopWaiting do
  begin
    { Process modem data }
    while Modem.Port.CharReady do
    begin
      { Read one byte }
      if Modem.ReadChar(CharRead) = S_FAIL then
      begin
        Result := S_FAIL;
        Exit;
      end;
      { Check for <DLE> codes }
      if CharRead = DLE then
      begin
        { Read DLE code }
        if Modem.ReadChar(CharRead) = S_FAIL then
        begin
          Result := S_FAIL;
          Exit;
        end;
        DebugMsgFmt(L_JUNK, SReadDLE, [CharRead]);
        { Handle DLE }
        Modem.HandleDLE(CharRead);
      end
      else
        DebugMsgFmt(L_WARN, SUnexpectedByte, [CharRead]);
    end;
    { Yield callback }
    YieldProc;
    { Check for new events }
    Modem.CheckEvents;
    { Check timeout }
    if (TimeOut > 0) and (Abs(Longint(GetTickCount) - StartTime) >= Timeout) and (not FStopWaiting) then
    begin
      WasTimeout := True;
      Modem.QueueEvent(TModemEvent.Create(OPERATION_TIMEOUT));
      Modem.StopWaiting;
    end;
    { Wait for data }
    SleepEx(Profile.PortReadDelay, False);
  end;
  { Process events queue }
  Modem.CheckEvents;
  { Check result }
  if WasTimeout then Result := E_TIMEOUT
  else if FStopWaiting then Result := E_INTERRUPTED
  else Result := S_OK;
end;

function TIS101.WaitAnswer(const ExpectedAnswers: string; Timeout: Longint): Integer;
var
  CharRead: Char;
  Buffer: string;
  FirstChar: Boolean;
  StartTime: Longint;
  WasTimeout: Boolean;
begin
  { Check modem specified }
  if not CheckModem('WaitAnswer', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  FStopWaiting := False;
  Buffer := '';
  FirstChar := True;
  StartTime := GetTickCount;
  WasTimeout := False;
  { Wait modem answer }
  while not FStopWaiting do
  begin
    { Check for modem char }
    if Modem.Port.CharReady then
    begin
      { Read one byte }
      if Modem.ReadChar(CharRead) = S_FAIL then
      begin
        Result := S_FAIL;
        Exit;
      end;
      { Check for <DLE> codes }
      if CharRead = DLE then
      begin
        { Read DLE code }
        if Modem.ReadChar(CharRead) = S_FAIL then
        begin
          Result := S_FAIL;
          Exit;
        end;
        DebugMsgFmt(L_JUNK, SReadDLE, [CharRead]);
        { Handle DLE }
        Modem.HandleDLE(CharRead);
      end
      { Check for valid chars }
      else if not (CharRead in [LF, CR, XON, XOFF]) then
      begin
        if FirstChar then
        begin
          Buffer := CharRead;
          FirstChar := False;
        end
        else
          Buffer := Buffer + CharRead
      end
      { Check for unexpected XON/XOFF codes }
      else if not (CharRead in [LF, CR]) then
        DebugMsgFmt(L_WARN, SUnexpectedByte, [CharRead])
      { Check for end of line }
      else if CharRead = LF then
      begin
        DebugMsgFmt(L_JUNK, SWaitAnswer, [Buffer]);
        { Analyze modem answer }
        if (not FirstChar) and (Length(Buffer) > 0) then
        begin
          Result := Modem.Analyze(Buffer, ExpectedAnswers);
          if Result <> VMA_FAIL then Exit;
        end;
        Buffer := '';
        FirstChar := True;
      end;
    end
    else
      { Wait for data }
      SleepEx(Profile.PortReadDelay, False);
    { Yield callback }
    YieldProc;
    { Check for new events }
    Modem.CheckEvents;
    { Check timeout }
    if (TimeOut > 0) and (Abs(Longint(GetTickCount) - StartTime) >= Timeout) and (not FStopWaiting) then
    begin
      WasTimeout := True;
      Modem.QueueEvent(TModemEvent.Create(OPERATION_TIMEOUT));
      Modem.StopWaiting;
    end;
  end;
  { Process events queue }
  Modem.CheckEvents;
  { Check result }
  if WasTimeout then Result := E_TIMEOUT
  else if FStopWaiting then Result := E_INTERRUPTED
  else Result := S_OK;
end;

initialization
  RegisterModemDriver(SIS101Name, TIS101);

end.
