{
  Voice Modem Library v1.0

  This file contains some classes to read and write modem params to
  the INI files and Windows registry.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_IniFile;

interface

uses
  Windows, SysUtils, Classes, IniFiles, Registry, VML_Modem;

type                 

{ INI file params reader }

  TModemIniReader = class(TModemReader)
  private
    FIniFile: TIniFile;
  public
    constructor Create(AIniFile: TIniFile);
    procedure ReadSection(const Section: string; Strings: TStrings); override;
    function ReadString(const Section, Ident, Default: string): string; override;
  end;

{ Registry params reader }

  TModemRegReader = class(TModemReader)
  private
    FRegIniFile: TRegIniFile;
  public
    constructor Create(ARegIniFile: TRegIniFile);
    procedure ReadSection(const Section: string; Strings: TStrings); override;
    function ReadString(const Section, Ident, Default: string): string; override;
  end;

{ INI file params writer }

  TModemIniWriter = class(TModemWriter)
  private
    FIniFile: TIniFile;
  public
    constructor Create(AIniFile: TIniFile);
    procedure DeleteKey(const Section, Ident: String); override;
    procedure EraseSection(const Section: string); override;
    procedure WriteString(const Section, Ident, Value: String); override;
  end;

{ Regisrty params writer }

  TModemRegWriter = class(TModemWriter)
  private
    FRegIniFile: TRegIniFile;
  public
    constructor Create(ARegIniFile: TRegIniFile);
    procedure DeleteKey(const Section, Ident: String); override;
    procedure EraseSection(const Section: string); override;
    procedure WriteString(const Section, Ident, Value: String); override;
  end;

{ Modem INI File }

  TModemIniFile = class(TIniFile)
  public
    function ReadDriver(Instance: TModemDriver): TModemDriver; virtual;
    procedure WriteDriver(Driver: TModemDriver); virtual;
  end;

{ Modem Registry INI File }

  TModemRegIniFile = class(TRegIniFile)
  public
    function ReadDriver(Instance: TModemDriver): TModemDriver; virtual;
    procedure WriteDriver(Driver: TModemDriver); virtual;
  end;

{ Driver ini files read/write routines }

function ReadDriverIniFile(const FileName: string; Instance: TModemDriver): TModemDriver;
procedure WriteDriverIniFile(const FileName: string; Driver: TModemDriver);

{ Profile ini files read/write routines }

procedure ReadProfileIniFile(const FileName: string; Profile: TModemProfile);
procedure WriteProfileIniFile(const FileName: string; Profile: TModemProfile);

{ Driver registry read/write routines }

function ReadDriverRegistry(RootKey: HKEY; const Section: string; Instance: TModemDriver): TModemDriver;
procedure WriteDriverRegistry(RootKey: HKEY; const Section: string; Driver: TModemDriver);

{ Profile registry read/write routines }

procedure ReadProfileRegistry(RootKey: HKEY; const Section: string; Profile: TModemProfile);
procedure WriteProfileRegistry(RootKey: HKEY; const Section: string; Profile: TModemProfile);

implementation

{ TModemIniReader }

constructor TModemIniReader.Create(AIniFile: TIniFile);
begin
  FIniFile := AIniFile;
end;

procedure TModemIniReader.ReadSection(const Section: string; Strings: TStrings);
begin
  FIniFile.ReadSection(Section, Strings);
end;

function TModemIniReader.ReadString(const Section, Ident, Default: string): string;
begin
  Result := FIniFile.ReadString(Section, Ident, Default);
end;

{ TModemRegReader }

constructor TModemRegReader.Create(ARegIniFile: TRegIniFile);
begin
  FRegIniFile := ARegIniFile;
end;

procedure TModemRegReader.ReadSection(const Section: string; Strings: TStrings);
begin
  FRegIniFile.ReadSection(Section, Strings);
end;

function TModemRegReader.ReadString(const Section, Ident, Default: string): string;
begin
  Result := FRegIniFile.ReadString(Section, Ident, Default);
end;

{ TModemIniWriter }

constructor TModemIniWriter.Create(AIniFile: TIniFile);
begin
  FIniFile := AIniFile;
end;

procedure TModemIniWriter.DeleteKey(const Section, Ident: String);
begin
  FIniFile.DeleteKey(Section, Ident);
end;

procedure TModemIniWriter.EraseSection(const Section: string);
begin
  FIniFile.EraseSection(Section);
end;

procedure TModemIniWriter.WriteString(const Section, Ident, Value: String);
var
  V: string;
  L: Integer;
begin
  { If the Value string is delimited by quotation marks, Windows discards the
    quotation marks when reads it from INI file. Doubling quotation marks
    includes quotation marks in the string. }
  V := Value;
  L := Length(V);
  if (L > 1) and (V[1] = V[L]) and (V[1] in ['"', '''']) then V := V[1] + V + V[1];
  { Write value }
  FIniFile.WriteString(Section, Ident, V);
end;

{ TModemRegWriter }

constructor TModemRegWriter.Create(ARegIniFile: TRegIniFile);
begin
  FRegIniFile := ARegIniFile;
end;

procedure TModemRegWriter.DeleteKey(const Section, Ident: String);
begin
  FRegIniFile.DeleteKey(Section, Ident);
end;

procedure TModemRegWriter.EraseSection(const Section: string);
begin
  FRegIniFile.EraseSection(Section);
end;

type
  THackRegIniFile = class(TRegIniFile);

procedure TModemRegWriter.WriteString(const Section, Ident, Value: String);
var
  Key, OldKey: HKEY;
begin
  with THackRegIniFile(FRegIniFile) do
  begin
    { TRegIniFile have a bug: if you put data to the section wich was
      created by another more provileged user you get exception. So check
      key exists before write string. }
    if not KeyExists(Section) then
    begin
      WriteString(Section, Ident, Value);
      Exit;
    end;
    { Write string }
    Key := GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      SetCurrentKey(Key);
      try
        PutData(Ident, PChar(Value), Length(Value), rdString);
      finally
        SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end;
  end;
end;

{ TModemIniFile }

function TModemIniFile.ReadDriver(Instance: TModemDriver): TModemDriver;
var
  Reader: TModemReader;
begin
  Reader := TModemIniReader.Create(Self);
  try
    Result := Reader.ReadDriver(Instance);
  finally
    Reader.Free;
  end;
end;

procedure TModemIniFile.WriteDriver(Driver: TModemDriver);
var
  Writer: TModemWriter;
begin
  Writer := TModemIniWriter.Create(Self);
  try
    Writer.WriteDriver(Driver);
  finally
    Writer.Free;
  end;
end;

{ TModemRegIniFile }

function TModemRegIniFile.ReadDriver(Instance: TModemDriver): TModemDriver;
var
  Reader: TModemReader;
begin
  Reader := TModemRegReader.Create(Self);
  try
    Result := Reader.ReadDriver(Instance);
  finally
    Reader.Free;
  end;
end;

procedure TModemRegIniFile.WriteDriver(Driver: TModemDriver);
var
  Writer: TModemWriter;
begin
  Writer := TModemRegWriter.Create(Self);
  try
    Writer.WriteDriver(Driver);
  finally
    Writer.Free;
  end;
end;

{ Driver ini file read/write routines }

function ReadDriverIniFile(const FileName: string; Instance: TModemDriver): TModemDriver;
var
  IniFile: TModemIniFile;
begin
  IniFile := TModemInifile.Create(FileName);
  try
    Result := IniFile.ReadDriver(Instance);
  finally
    IniFile.Free;
  end;
end;

procedure WriteDriverIniFile(const FileName: string; Driver: TModemDriver);
var
  IniFile: TModemIniFile;
begin
  IniFile := TModemInifile.Create(FileName);
  try
    IniFile.WriteDriver(Driver);
  finally
    IniFile.Free;
  end;
end;

{ Profile ini file read/write routines }

procedure ReadProfileIniFile(const FileName: string; Profile: TModemProfile);
var
  IniFile: TModemIniFile;
  Reader: TModemReader;
begin
  IniFile := TModemInifile.Create(FileName);
  try
    Reader := TModemIniReader.Create(IniFile);
    try
      Profile.ReadData(Reader);
    finally
      Reader.Free;
    end;
  finally
    IniFile.Free;
  end;
end;

procedure WriteProfileIniFile(const FileName: string; Profile: TModemProfile);
var
  IniFile: TModemIniFile;
  Writer: TModemWriter;
begin
  IniFile := TModemInifile.Create(FileName);
  try
    Writer := TModemIniWriter.Create(IniFile);
    try
      Profile.WriteData(Writer);
    finally
      Writer.Free;
    end;
  finally
    IniFile.Free;
  end;
end;

{ Driver registry read/write routines }

function ReadDriverRegistry(RootKey: HKEY; const Section: string; Instance: TModemDriver): TModemDriver;
var
  RegIniFile: TModemRegIniFile;
begin
  RegIniFile := TModemRegIniFile.Create('');
  try
    RegIniFile.RootKey := RootKey;
    RegIniFile.OpenKey(Section, True);
    Result := RegIniFile.ReadDriver(Instance);
  finally
    RegIniFile.Free;
  end;
end;

procedure WriteDriverRegistry(RootKey: HKEY; const Section: string; Driver: TModemDriver);
var
  RegIniFile: TModemRegIniFile;
begin
  RegIniFile := TModemRegIniFile.Create('');
  try
    RegIniFile.RootKey := RootKey;
    RegIniFile.OpenKey(Section, True);
    RegIniFile.WriteDriver(Driver);
  finally
    RegIniFile.Free;
  end;
end;

{ Profile registry read/write routines }

procedure ReadProfileRegistry(RootKey: HKEY; const Section: string; Profile: TModemProfile);
var
  RegIniFile: TModemRegIniFile;
  Reader: TModemReader;
begin
  RegIniFile := TModemRegIniFile.Create('');
  try
    RegIniFile.RootKey := RootKey;
    RegIniFile.OpenKey(Section, True);
    Reader := TModemRegReader.Create(RegIniFile);
    try
      Profile.ReadData(Reader);
    finally
      Reader.Free;
    end;
  finally
    RegIniFile.Free;
  end;
end;

procedure WriteProfileRegistry(RootKey: HKEY; const Section: string; Profile: TModemProfile);
var
  RegIniFile: TModemRegIniFile;
  Writer: TModemWriter;
begin
  RegIniFile := TModemRegIniFile.Create('');
  try
    RegIniFile.RootKey := RootKey;
    RegIniFile.OpenKey(Section, True);
    Writer := TModemRegWriter.Create(RegIniFile);
    try
      Profile.WriteData(Writer);
    finally
      Writer.Free;
    end;
  finally
    RegIniFile.Free;
  end;
end;

end.
