{
  Voice Modem Library v1.0

  This file contains hardware driver functions for
  some USRobotics modems.

  Based on US_Robotics driver, originally from
  Steven Wormley <wormley@step.mother.com>

  Known bugs:
    - My USRobotics Sportster Flash doesn't catch dtmf codes after a
      beep. If I send it the "AT#VLS=4A" (answer phone) string after an
      "AT#VTS[%d,0,%d]" (beep) command it would resume detecting tones.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_USRobotics;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, VML_Modem, VML_Consts, VML_IS101;

const

{ US Robotics available compression modes }

  USROBOTICS_GSM        = 1;
  USROBOTICS_ADPCM_2    = 2;
  USROBOTICS_ADPCM_3    = 3;
  USROBOTICS_ADPCM_4    = 4;
  USROBOTICS_IMAADPCM_2 = 5;
  USROBOTICS_IMAADPCM_3 = 6;
  USROBOTICS_IMAADPCM_4 = 7;

{ US Robotics modem strings }

  SUSRoboticsName    = 'US Robotics';
  SUSRoboticsRMDName = 'US Robotics';

{$I *.INC}

type

{ TUSRobotics }

  TUSRobotics = class(TIS101)
  public
    constructor Create(AOwner: TComponent); override;
    function GetVoiceLen(Compression, Speed, Bits: Integer; DataSize: Longint): Integer; override;
    function HandleDLE(Code: Char): Integer; override;
  end;

implementation

uses
  MMReg;

{ TUSRobotics }

constructor TUSRobotics.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { Modem name }
  ModemName := SUSRoboticsName;
  ModemRMDName := SUSRoboticsRMDName;
  { &H1 - Transmit data flow control: Clear to Send (CTS).
    &R2 - Receive data flow control: only Request to Send (RTS).
    &I0 - Disable software flow control. }
  HardFlowCmd := 'AT&H1&R2&I0';
  SoftFlowCmd := '&H2&R1&I2';
  { #VLS=0 - Select the telephone line.
    A      - Answering in Voice. }
  PickPhoneCmd := 'AT#VLS=0A';
  PickPhoneAnsw := 'VCON';
  { #VTS - Generate Tone Signals. }
  BeepCmd := 'AT#VTS=[%d,0,%d]';
  BeepTimeUnit := 100;
  { #VTX - Voice Transmit. }
  StartPlayCmd := 'AT#VTX';
  IntrPlayCmd := '<DLE><CAN>';
  IntrPlayAnsw := 'OK|VCON';
  StopPlayAnsw := 'OK|VCON';
  { #VRX - Voice Receive. }
  StartRecCmd := 'AT#VRX';
  StopRecCmd := '<DLE>';
  StopRecAnsw := 'OK|VCON';
  { #CLS - Select Data, Fax, or Voice in any mode. }
  SwitchModeCmd := 'AT#CLS=%s';
  VoiceModeOnCmd := 'AT#CLS=8';
  VoiceModeOffCmd := 'AT#CLS=0';
  { #VTS - Generate DTMF tone. }
  PlayDTMFCmd := 'AT#VTS=%s';
  { S30=n - Deadman (Inactivity) Timer. }
  InactivityTimerCmd := 'ATS30=%d';
  InactivityTimerAnsw := 'OK';
  { #BDR - Select a specific DTE/modem baud rate. }
  BaudRateCmd := 'AT#BDR=%d';
  BaudRateAnsw := 'OK';
  BaudRateUnit := 2400;
  { #VSS - Silence Detection Tuner.
    #VSP - Silence Detection Period. }
  SilenceDetectCmd := 'AT#VSS=%d||AT#VSP=%d';
  SilenceDetectAnsw := 'OK||OK';
  SilenceSensMin := 0;
  SilenceSensMax := 3;
  SilenceDeleteCmd := 'AT#VSD=%d';
  SilenceDeleteAnsw := 'OK';
  { #VGT - Set the sensitivity of the microphone in speaker phone mode. }
  TransmitGainCmd := 'AT#VGT=%d';
  TransmitGainAnsw := 'OK';
  TransmitGainMin := 0;
  TransmitGainMax := 255;
  { #VGR - Set the speaker volume in speaker phone mode. }
  ReceiveGainCmd := 'AT#VGR=%d';
  ReceiveGainAnsw := 'OK';
  ReceiveGainMin := 0;
  ReceiveGainMax := 255;
  { #VRA - Ringback Goes Away Timer.
    #VRN - Ringback Never Came Timer. }
  RingbackTimersCmd := 'AT#VRA=%d#VRN=%d';
  RingbackTimersAnsw := 'OK';
  { #VTD - DTMF Tone Reporting. }
  DTMFDetectCmd := 'AT#VTD=3F,3F,3F';
  DTMFDetectAnsw := 'OK';
  { #CID - Enable Caller ID detection and Select Reporting Format. }
  CallerIDOnCmd := 'AT#CID=1';
  CallerIDOnAnsw := 'OK';
  CallerIDOffCmd := 'AT#CID=0';
  CallerIDOffAnsw := 'OK';
  CallerIDFormat := 'NMBR =';
  { S41=1 - Enable distincitve ring type. }
  DRingOnCmd := 'ATS41=1';
  DRingOnAnsw := 'OK';
  DRingOffCmd := 'ATS41=0';
  DRingOffAnsw := 'OK';
  { "DATE = " - Caller ID response.
    "TIME = " - Caller ID response.
    "NAME = " - Caller ID response.
    "MESG = " - Caller ID response. }
  Responses.Add('DATE =', VMA_IGNORE);
  Responses.Add('TIME =', VMA_IGNORE);
  Responses.Add('NAME =', VMA_IGNORE);
  Responses.Add('MESG =', VMA_IGNORE);
  { USRobotics receive <DLE>t event when local handset goes off hook but
    no <DLE>H or <DLE>P. }
  DLECodes.Add('t', HANDSET_OFF_HOOK, #0, '');
  { #VLS=0 - Telephone line.
    #VLS=1 - Local handset.
    #VLS=2 - External speaker.
    #VLS=3 - Internal microphone.
    #VLS=4 - Internal speaker.
    #VLS=6 - Speakerphone. }
  Devices.Clear;
  Devices.Add(NO_DEVICE, SNoDevice, 'AT#VLS=0H0', 'OK|VCON', DF_HIDDEN);
  Devices.Add(DIALUP_LINE, SDialupLine, 'AT#VLS=0', 'OK|VCON', DF_TRANSMIT_RECEIVE);
  Devices.Add(INTERNAL_MICROPHONE, SInternalMicrophone, 'AT#VLS=3','OK|VCON', DF_RECEIVE);
  Devices.Add(EXTERNAL_SPEAKER, SExternalSpeaker, 'AT#VLS=2','OK|VCON', DF_TRANSMIT);
  Devices.Add(INTERNAL_SPEAKER, SInternalSpeaker, 'AT#VLS=4','OK|VCON', DF_TRANSMIT);
  Devices.Add(LOCAL_HANDSET, SLocalHandset, 'AT#VLS=1','OK|VCON', DF_TRANSMIT_RECEIVE);
  Devices.Add(SPEAKER_PHONE, SSPeakerPhone, 'AT#VLS=6','OK|VCON', DF_TRANSMIT_RECEIVE);
  { #VSM=128,8000 - GSM/USR 8000 Hz (default).
    #VSM=129,8000 - ADPCM 2,3,4 bits per sample 8000 Hz.
    #VSM=130,8000 - IMA ADPCM 2,3,4 bits per sample 8000 Hz.
    #VBS=n        - Sample rate selection (ADPCM only). }
  Compressions.Clear;
  Compressions.Add(USROBOTICS_GSM, SUSRGSM, 'AT#VSM=128,8000', 'OK', 8000, 8, CF_8000 or CF_DEFAULT, 0, 0, 0);
  Compressions.Add(USROBOTICS_ADPCM_2, SUSRADPCM2, 'AT#VSM=129,8000||AT#VBS=2', 'OK||OK', 8000, 2, CF_8000, 0, 0, 0);
  Compressions.Add(USROBOTICS_ADPCM_3, SUSRADPCM3, 'AT#VSM=129,8000||AT#VBS=3', 'OK||OK', 8000, 3, CF_8000, 0, 0, 0);
  Compressions.Add(USROBOTICS_ADPCM_4, SUSRADPCM4, 'AT#VSM=129,8000||AT#VBS=4', 'OK||OK', 8000, 4, CF_8000, 0, 0, 0);
  Compressions.Add(USROBOTICS_IMAADPCM_2, SUSRIMAADPCM2, 'AT#VSM=130,8000||AT#VBS=2', 'OK||OK', 8000, 2, CF_8000, 0, 0, 0);
  Compressions.Add(USROBOTICS_IMAADPCM_3, SUSRIMAADPCM3, 'AT#VSM=130,8000||AT#VBS=3', 'OK||OK', 8000, 3, CF_8000, 0, 0, 0);
  Compressions.Add(USROBOTICS_IMAADPCM_4, SUSRIMAADPCM4, 'AT#VSM=130,8000||AT#VBS=4', 'OK||OK', 8000, 4, CF_8000, WAVE_FORMAT_IMA_ADPCM, 8000, CF_8000 or CF_DEFAULT);
end;

function TUSRobotics.GetVoiceLen(Compression, Speed, Bits: Integer; DataSize: Longint): Integer;
var
  BPS: Integer;
begin
  if Compression = USROBOTICS_GSM then
  begin
    { This formula correct only for encapsulated GSM frame format }
    BPS := Speed * 38 div 160;
    { Check BPS value and return duration of voice message }
    if BPS > 0 then
    begin
      Result := Round(DataSize / BPS * 1000);
      Exit;
    end;
  end;
  { Use default calculation from IS101 }
  Result := inherited GetVoiceLen(Compression, Speed, Bits, DataSize);
end;

function TUSRobotics.HandleDLE(Code: Char): Integer;
begin
  { Check modem specified }
  if not CheckModem('HandleDLE', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Handle DLE codes }
  case Code of
     'd', 'i':
      { In this situation IS101.HandleDLE will create a DAIL_TONE event.
        This DLE is however incorrectly generated by the USRobotics modems
        and should be ignored when we are either PLAYING or RECORDING. }
      case Modem.State of
        PLAYING:
          begin
            DebugMsgFmt(L_JUNK, SIgnoreDLE, [Code, 'PLAYING']);
            Result := S_OK;
            Exit;
          end;
        RECORDING:
          begin
            DebugMsgFmt(L_JUNK, SIgnoreDLE, [Code, 'RECORDING']);
            Result := S_OK;
            Exit;
          end;
      end;
  end;
  { Standard handling }
  Result := inherited HandleDLE(Code);
end;

initialization
  RegisterModemDriver(SUSRoboticsName, TUSRobotics);

end.
